/*
 * Copyright 2012 Olivier Godineau
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package olg.csv.base;

import java.io.IOException;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * A "Proxy" writer which Provides a mecanism to skip empty row.
 * 
 * @author Olivier Godineau
 * 
 */
public class IgnoreNullWriter implements IWriter {
	/**
	 * the class logger.
	 */
	private static final Logger LOGGER = LoggerFactory.getLogger(IgnoreNullWriter.class);
	/**
	 * the original writer.
	 */
	private final IWriter writer;
	/**
	 * num of the next row to write.
	 */
	private int nextRowNum = 1;

	/**
	 * 
	 * @param writer
	 *            a concret reader
	 */
	public IgnoreNullWriter(IWriter writer) {
		super();
		if (writer == null) {
			throw new IllegalArgumentException(IgnoreNullWriter.class.getSimpleName()
					+ " Constructor IWriter argument must not be null");
		}
		this.writer = writer;

	}

	/**
	 * Close the original writer.
	 */
	public void close() {
		try {
			writer.close();
		} catch (IOException e) {
			LOGGER.info("Error on closing reader", e);
		}
	}

	/**
	 * writes a row only if not empty.
	 * 
	 * @param row
	 *            the row.
	 */
	public void addRow(Row row) {

		if (isValid(row)) {
			Row copy = row.copy(nextRowNum++);
			writer.addRow(copy);
		}
	}

	/**
	 * Write a row only if not empty.
	 * 
	 * @param values
	 *            the row.
	 */
	public void addLine(String[] values) {
		if (isValid(values)) {
			writer.addLine(values);
			nextRowNum++;
		}

	}

	/**
	 * @return the original withHeaders attribute.
	 */
	public boolean isWithHeaders() {
		return writer.isWithHeaders();
	}

	/**
	 * Check if a row is not null and not empty.
	 * 
	 * @param row
	 *            the row
	 * @return true if the row is not null and not empty.
	 */
	private boolean isValid(Row row) {
		return (row != null && !row.isEmpty());

	}

	/**
	 * Check if a row is not null and not empty.
	 * 
	 * @param values
	 *            the row
	 * @return true if the row is not null and not empty.
	 */
	private boolean isValid(String[] values) {
		boolean retour = false;
		if (values != null) {

			for (String value : values) {
				if (value != null) {
					retour = true;
					break;
				}
			}
		}
		return retour;
	}

}
