/*
 * Copyright 2012 Olivier Godineau
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package olg.csv.base.csv;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;
import java.util.NoSuchElementException;

import olg.csv.base.Cell;
import olg.csv.base.IReader;
import olg.csv.base.ReaderException;
import olg.csv.base.Row;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * CSV Reader. CSV Implementation of IReader.
 * 
 * 
 */
public final class CSVReader implements IReader {

	/**
	 * the class Logger.
	 */
	private static final Logger LOGGER = LoggerFactory.getLogger(CSVReader.class);

	/**
	 * if true the returned rows will only contain not empty cells.
	 */
	private boolean skipEmptyCell = false;

	/**
	 * cell separator.
	 */
	private char separator;

	/**
	 * Cell delimiter.
	 */
	private char delimiter;

	/**
	 * Double Delimiter. Virtue of CSV rfc, delimiter character must be twice in
	 * a cell to represent it.
	 */
	private String ddelimiter;

	/**
	 * Read file Charset.
	 */
	private String charset;

	/**
	 * to skip headers.
	 */
	private boolean withHeaders;

	/**
	 * Current cell number.
	 */
	private int fieldIndex = 0;

	/**
	 * current row number.
	 */
	private int recordIndex = 1;

	/**
	 * the CSV file input stream.
	 */
	private FileInputStream fileInputStream = null;

	/**
	 * the char search reader applied on the stream.
	 */
	private CharSearchReader reader = null;

	/**
	 * Read Row size.
	 */
	private int rowSize;

	/**
	 * Constructor with custom settings.
	 * <p>
	 * throws UnsupportedCharsetException if charset is not supported.
	 * </p>
	 * <p>
	 * throws FileNotFoundException if the file does not exist, is a directory
	 * rather than a regular file, or for some other reason cannot be opened for
	 * reading.
	 * </p>
	 * <p>
	 * throws IllegalArgumentException if file, CSVSettings, lineSeparator
	 * setting or charsetName setting is <code>null</code>
	 * </p>
	 * 
	 * @param file
	 *            the file to be read.
	 * @param settings
	 *            the settings.
	 * @throws IOException
	 *             if error occurs on file opening.
	 * 
	 * @see CSVSettings to see default parameters values
	 */
	public CSVReader(File file, CSVSettings settings) throws IOException {
		if (file == null) {
			throw new IllegalArgumentException("CSVReader Constructor file argument must not be null");
		}
		if (settings == null) {
			throw new IllegalArgumentException();
		}

		if (settings.getCharsetName() == null) {
			throw new IllegalArgumentException(
					"CSVReader Constructor settings argument  charset attribute must not be null");
		}
		this.delimiter = settings.getDelimiter();
		this.ddelimiter = new String(new char[]{delimiter, delimiter});
		this.separator = settings.getSeparator();
		this.charset = settings.getCharsetName();
		this.withHeaders = settings.isWithHeaders();

		this.skipEmptyCell = settings.isSkipEmptyCell();
		this.fileInputStream = new FileInputStream(file);
		this.reader = new CharSearchReader(fileInputStream, charset);

	}

	/**
	 * Constructor with custom settings.
	 * <p>
	 * throws UnsupportedCharsetException if charset is not supported.
	 * </p>
	 * <p>
	 * throws FileNotFoundException if the file does not exist, is a directory
	 * rather than a regular file, or for some other reason cannot be opened for
	 * reading.
	 * </p>
	 * 
	 * @param file
	 *            the file.
	 * @param settings
	 *            the settings
	 * @param bufSize
	 *            the buffer size.
	 * @throws IOException
	 *             if error occurs on file opening.
	 * 
	 * @see CSVSettings to see default parameters values
	 */
	protected CSVReader(File file, CSVSettings settings, int bufSize) throws IOException {

		this.delimiter = settings.getDelimiter();
		this.ddelimiter = new String(new char[]{delimiter, delimiter});
		this.separator = settings.getSeparator();
		this.charset = settings.getCharsetName();
		this.withHeaders = settings.isWithHeaders();
		this.skipEmptyCell = settings.isSkipEmptyCell();
		this.fileInputStream = new FileInputStream(file);
		this.reader = new CharSearchReader(fileInputStream, charset, bufSize);

	}

	/**
	 * Constructor with custom settings.
	 * <p>
	 * throws UnsupportedCharsetException if charset is not supported.
	 * </p>
	 * <p>
	 * throws IllegalArgumentException if input, CSVSettings or charsetName
	 * setting is <code>null</code>
	 * </p>
	 * 
	 * @param input
	 *            the stream
	 * @param settings
	 *            the settings.
	 * @throws IOException
	 *             if error occurs on stream buffering.
	 * 
	 * @see CSVSettings to see default parameters values
	 */
	public CSVReader(InputStream input, CSVSettings settings) throws IOException {
		if (input == null) {
			throw new IllegalArgumentException("CSVReader Constructor InputStream argument must not be null");
		}
		if (settings == null) {
			throw new IllegalArgumentException();
		}

		if (settings.getCharsetName() == null) {
			throw new IllegalArgumentException(
					"CSVReader Constructor settings argument  charset attribute must not be null");
		}
		this.delimiter = settings.getDelimiter();
		this.ddelimiter = new String(new char[]{delimiter, delimiter});
		this.separator = settings.getSeparator();
		this.charset = settings.getCharsetName();
		this.withHeaders = settings.isWithHeaders();
		this.skipEmptyCell = settings.isSkipEmptyCell();

		this.reader = new CharSearchReader(input, charset);

	}

	/**
	 * Indicates the CSV file has a header line.
	 * 
	 * @return if the CSV file has a header line.
	 */
	public boolean isWithHeaders() {
		return this.withHeaders;
	}

	/**
	 * {@inheritDoc}
	 */
	public boolean hasNext() {
		return reader.exists(0) && !reader.isEof();

	}
	/**
	 * {@inheritDoc}
	 */
	public Row next() {
		if (!hasNext()) {
			throw new NoSuchElementException();
		}

		return setNext();
	}

	/**
	 * {@inheritDoc} CSVReader throws an UnsupportedOperationException.
	 */
	public void remove() {
		throw new UnsupportedOperationException();

	}

	/**
	 * Closes this stream and catches and logs IOException. if the parent stream
	 * is provided and passed to CSVReader constructor by user, user must close
	 * it (closes what you opens!)
	 */
	public void close() {
		if (reader != null) {

			reader.close();

		}
		if (fileInputStream != null) {
			try {
				fileInputStream.close();
			} catch (IOException e) {
				LOGGER.info("Error on closing CVSReader fileInputStream", e);
			}
		}

	}

	/**
	 * Returns next row.
	 * <p>
	 * Throws a ReaderException if a CSV constraint is not satisfied.
	 * </p>
	 * 
	 * @return the next row.
	 */
	private Row setNext() {
		Row retour = null;
		try {

			int size;
			int rowNum;
			List<Cell> cells = null;
			rowNum = recordIndex;
			size = 0;
			cells = new ArrayList<Cell>();

			while (rowNum == recordIndex && !reader.isEof()) {
				Cell cell = readCell();
				size++;
				if (cell.getValue() != null || !skipEmptyCell) {
					cells.add(cell);
				}
			}
			retour = new Row(rowNum, cells, size);

			rowSize = (rowSize == 0 ? size : rowSize);
			if (rowSize != size) {
				throw new ReaderException("Row[" + rowNum + "] size[" + size + "] differs from the expected size :"
						+ rowSize);
			}

		} catch (IOException ex) {
			throw new ReaderException("Read Error at line :" + recordIndex + " and field :" + fieldIndex, ex);
		}
		return retour;
	}

	/**
	 * Read a cell.
	 * <p>
	 * Throws a ReaderException if a CSV constraint is not satisfied.
	 * </p>
	 * 
	 * @return the cell.
	 * @throws IOException
	 *             if I/O error occurs on stream reading.
	 */
	private Cell readCell() throws IOException {

		Cell retour = null;

		// Test sur la longueur du buf
		if (!reader.isEof()) {
			String champ = null;
			int fieldNum = fieldIndex;

			if (reader.charAt(0) == delimiter) {

				champ = getFieldDelimiterCase();
			} else { // cas sans délimiteur de champ
				champ = getFieldNoDelimiterCase();
			}
			if ("".equals(champ)) {
				retour = new Cell(fieldNum, null);
			} else {
				retour = new Cell(fieldNum, champ);
			}

		}

		return retour;
	}

	/**
	 * Get next field value in case field has delimiters around.
	 * <p>
	 * Throws a ReaderException if a CSV constraint is not satisfied.
	 * </p>
	 * 
	 * @return the value.
	 * @throws IOException
	 *             if I/O error occurs on stream reading.
	 */
	private String getFieldDelimiterCase() throws IOException {
		String field = null;
		int index = 0;
		char nextCar;

		// Cas où EOF!?
		do {
			index = reader.findNextAtIndex(index + 1, delimiter) + 1;
			if (index == -1) {
				throw new ReaderException(String.format(" %s expected at line %s", delimiter, recordIndex));
			}
		} while (reader.exists(index) && reader.charAt(index) == delimiter && !reader.isEof());

		field = reader.substring(1, index - 1);
		reader.skip(index);

		if (reader.exists(0)) {
			nextCar = reader.charAt(0);
			reader.skip(1);

			if (nextCar == separator) {
				fieldIndex++;
			} else if (nextCar == '\r') {

				if (reader.exists(0) && reader.charAt(0) == '\n') {
					reader.skip(1);
				}
				fieldIndex = 0;
				recordIndex++;
			} else if (nextCar == '\n') {
				fieldIndex = 0;
				recordIndex++;
			} else {
				throw new ReaderException(String.format("%s, CR or LF expected, found : %s", separator, nextCar));
			}
		}

		return field.replaceAll(ddelimiter, String.valueOf(delimiter));

	}

	/**
	 * Get next field value when a field has no delimiters around.
	 * 
	 * @return the value.
	 * @throws IOException
	 *             if I/O error occurs on stream reading.
	 */
	private String getFieldNoDelimiterCase() throws IOException {

		String field = null;

		int index = reader.find(separator, '\n', '\r');
		if (index > -1) {
			char car = reader.charAt(index);
			field = reader.substring(0, index);
			// On recale le flu;
			reader.skip(index + 1);

			if (car == separator) {
				fieldIndex++;
			} else if (car == '\r') {

				if (reader.exists(0) && reader.charAt(0) == '\n') {
					reader.skip(1);
				}
				fieldIndex = 0;
				recordIndex++;
			} else if (car == '\n') {
				fieldIndex = 0;
				recordIndex++;
			} else {
				throw new ReaderException(String.format("%s, CR or LF expected, found : %s", separator, car));
			}

		} else {
			// EOF
			field = reader.substring(0, reader.getSize());

		}

		return field;
	}

}
