/*
 * Copyright 2012 Olivier Godineau
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package olg.csv.base.csv;

import java.io.BufferedReader;
import java.io.Closeable;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * This class provides methods based on the character search and hides the
 * original stream reading.
 * 
 * @author Olivier Godineau
 * 
 */
public class CharSearchReader implements Closeable {
	/**
	 * the class logger.
	 */
	private static final Logger LOGGER = LoggerFactory.getLogger(CharSearchReader.class);
	/**
	 * Reader Buffer size default size.
	 */
	public static final int BUF_SIZE = 512;
	/**
	 * Current buffer size.
	 */
	private int currentBufSize = 0;
	/**
	 * The input reader.
	 */
	private BufferedReader reader = null;
	/**
	 * Indicates if end of file is reached.
	 */
	private boolean eof = false;

	/**
	 * buffer offset.
	 */
	private int offset = 0;

	/**
	 * reader buffer size.
	 */
	private int maxBufSize;

	/**
	 * the char buffer.
	 */
	private final char[] cbuf;

	/**
	 * Retains the characters previously read into the buffer.
	 */
	private final StringBuilder store;

	/**
	 * 
	 * @param reader
	 *            the input stream.
	 * @param charset
	 *            the charset name.
	 * @throws IOException
	 *             If an I/O error occurs or If the named charset is not
	 *             supported
	 */
	public CharSearchReader(InputStream reader, String charset) throws IOException {
		this(reader, charset, BUF_SIZE);

	}

	/**
	 * 
	 * @param reader
	 *            the input stream.
	 * @param charset
	 *            the charset name.
	 * @param bufSize
	 *            the max buf size.
	 * @throws IOException
	 *             If an I/O error occurs or If the named charset is not
	 *             supported
	 */
	protected CharSearchReader(InputStream reader, String charset, int bufSize) throws IOException {
		super();
		this.maxBufSize = bufSize;
		this.reader = new BufferedReader(new InputStreamReader(reader, charset));
		this.cbuf = new char[this.maxBufSize];
		this.store = new StringBuilder(this.maxBufSize);
		loadBuf();

	}

	/**
	 * Indicates if end of file is reached.
	 * 
	 * @return eof;
	 */
	public boolean isEof() {
		return eof;
	}

	/**
	 * Returns the char at an index. Must be >=0.
	 * 
	 * @param index
	 *            the index.
	 * @return the char at this index.
	 * @throws IOException
	 *             when the index is not reachable or If an I/O error occurs
	 */
	public char charAt(int index) throws IOException {
		if (index < 0) {
			throw new IllegalArgumentException("index out of range:" + index);
		}
		char car;
		if (store.length() > index) {
			car = this.store.charAt(index);
		} else {
			while (!(index - store.length() < currentBufSize - offset)) {
				loadNextSequence();
			}
			car = cbuf[index - store.length() + offset];

		}

		return car;
	}

	/**
	 * Indicates if a char exists at an index in this stream.
	 * 
	 * @param index
	 *            the index
	 * @return true if a char exists at this index
	 */
	public boolean exists(int index) {
		if (index < 0) {
			throw new IllegalArgumentException("index out of range:" + index);
		}
		if (store.length() > index) {
			return true;
		} else {
			boolean reached;
			while (!(reached = index - store.length() < currentBufSize - offset) && !eof) {
				try {
					loadNextSequence();
				} catch (IOException e) {
					// Ne doit pas arriver
					LOGGER.warn("BUG on CharSearchReader.exists", e);

				}
			}
			return reached;

		}
	}

	/**
	 * Finds the first index of the first character found.
	 * 
	 * @param args
	 *            the chars to search.
	 * @return the index of the first occurrence of the first character found,
	 *         or -1 if any character does not occur.
	 * @throws IOException
	 *             the eof is reached without find chars or if an I/O error
	 *             occurs.
	 */
	public int find(char... args) throws IOException {

		return findNextAtIndexFromBegin(0, args);
	}

	/**
	 * Finds the first index of the first character found starting at a specific
	 * index in the stream.
	 * 
	 * @param begin
	 *            the index to start the search from.
	 * 
	 * @param args
	 *            the chars to search.
	 * @return the index of the first occurrence of the first character found
	 *         greater than or equal to begin, or -1 if any character does not
	 *         occur.
	 * @throws IOException
	 *             when the begin index is not reachable or the eof is reached
	 *             without find chars.
	 */
	public int findNextAtIndex(int begin, char... args) throws IOException {
		// to be certain begin is reached.
		charAt(begin);
		return findNextAtIndexFromBegin(begin, args);
	}

	/**
	 * Skips characters in the stream.
	 * 
	 * @param nb
	 *            The number of characters to skip
	 */
	public void skip(int nb) {
		if (nb < 0) {
			throw new IllegalArgumentException("skip value is negative");
		}
		if (nb > store.length() + currentBufSize - offset) {
			throw new IllegalArgumentException("skip value is  out of range");
		}
		if (nb < store.length()) {
			store.delete(0, nb);
		} else {
			offset += nb - store.length();
			store.delete(0, store.length());
		}
	}

	/**
	 * Returns a new String that contains a subsequence of characters currently
	 * contained in the stream.
	 * 
	 * @param begin
	 *            The beginning index, inclusive.
	 * @param end
	 *            The ending index, exclusive.
	 * @return the new String.
	 */
	public String substring(int begin, int end) {
		if (begin < 0) {
			throw new IllegalArgumentException("index out of range:" + begin);
		}
		if (end < 0) {
			throw new IllegalArgumentException("index out of range:" + end);
		}
		if (end > store.length() + currentBufSize - offset) {
			throw new IllegalArgumentException("index out of range:" + end);
		}
		if (begin > end) {
			throw new IllegalArgumentException("index out of range:" + (end - begin));
		}
		if (begin < store.length() && end < store.length()) {
			return store.substring(begin, end);
		}

		if (begin < store.length()) {
			return new StringBuilder(store.substring(begin)).append(cbuf, offset, end + offset - store.length())
					.toString();

		}

		return new String(cbuf, offset + begin, end - begin);

	}

	/**
	 * Returns the current size of the stream.
	 * 
	 * @return the current size.
	 */
	public int getSize() {
		return store.length() + currentBufSize - offset;
	}

	/**
	 * {@inheritDoc}
	 */
	public void close() {

		if (reader != null) {
			try {
				reader.close();
			} catch (IOException e) {
				LOGGER.info("Error on closing StringReader inputStreamReader", e);
			}
		}

	}

	/**
	 * Finds the first index of the first character found starting at a specific
	 * index in the buf.
	 * 
	 * @param begin
	 *            the index to start the search from.
	 * @param args
	 *            the chars to search.
	 * @return the index of the first occurrence of the first character found
	 *         greater than or equal to begin, or -1 if any character does not
	 *         occur.
	 */
	private int findNextInBufAtIndex(int begin, char... args) {
		int index = -1;
		search : for (int i = begin; i < currentBufSize; i++) {
			for (char arg : args) {
				if (cbuf[i] == arg) {
					index = i;
					break search;
				}
			}
		}
		return index;
	}

	/**
	 * Finds the first index of the first character found starting at a specific
	 * index in the stream.
	 * 
	 * @param begin
	 *            the index to start the search from.
	 * 
	 * @param args
	 *            the chars to search.
	 * @return the index of the first occurrence of the first character found
	 *         greater than or equal to begin, or -1 if any character does not
	 *         occur.
	 * @throws IOException
	 *             the eof is reached without find chars or if an I/O error
	 *             occurs.
	 */
	private int findNextAtIndexFromBegin(int begin, char... args) throws IOException {

		int index = -1;
		if (begin < store.length()) {
			for (char arg : args) {
				int indexStore = store.indexOf(String.valueOf(arg), begin);
				if (indexStore != -1) {
					index = (index == -1 ? indexStore : Math.min(index, indexStore));
				}
			}
		}
		if (index == -1) {
			// on recherche dans cbuf à partir de begin -store.length +offset
			// search on cbuf
			int beginBuf = Math.max(offset, begin - store.length() + offset);
			while ((index = findNextInBufAtIndex(beginBuf, args)) == -1 && !isEof()) {
				loadNextSequence();
				beginBuf = 0;
			}
			if (index != -1) {
				index += store.length() - offset;
			}
		}
		return index;
	}

	/**
	 * Stores the current sequence and loads the next sequence.
	 * 
	 * @throws IOException
	 *             if end of line is reached on loading the next sequence or an
	 *             I/O error occurs
	 */
	private void loadNextSequence() throws IOException {
		store.append(cbuf, offset, currentBufSize - offset);
		if (eof) {
			throw new IOException("End of file");
		}
		loadBuf();
	}

	/**
	 * Loads the next sequence.
	 * 
	 * @throws IOException
	 *             If an I/O error occurs
	 */
	private void loadBuf() throws IOException {
		currentBufSize = reader.read(cbuf);
		offset = 0;
		if (currentBufSize == -1) {
			eof = true;
			currentBufSize = 0;
		}
	}

}
