/*
 * Copyright 2012 Olivier Godineau
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package olg.csv.bean;

import java.io.Closeable;
import java.io.File;
import java.io.IOException;

import olg.csv.base.IWriter;
import olg.csv.base.IgnoreNullWriter;
import olg.csv.base.Row;
import olg.csv.bean.impl.RowProcessor;
import olg.csv.bean.loader.CellProcessorLoader;
import olg.csv.bean.loader.LoadException;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * Class specialized in writing objects into a file Used a IWriter to write a
 * list of Strings into a file Uses a list of PropertyFormatter to transform an
 * <code>E</code> object into a list of Strings.
 * 
 * @author Olivier Godineau
 * 
 * @param <E>
 *            object this writer deals with.
 * @see IWriter
 * @see olg.csv.bean.impl.PropertyFormatter
 */
public class BeanWriter<E> implements Closeable {
	/**
	 * The class logger.
	 */
	private static final Logger LOGGER = LoggerFactory.getLogger(BeanWriter.class);
	/**
	 * the writer. Allows to write a list of Strings on a dedicated output.
	 */
	private final IWriter baseWriter;

	/**
	 * The row processor.
	 */
	private final IRowProcessor<E> rowProcessor;

	/**
	 * 
	 * @param configFile
	 *            the file which describes how to load the field formatters used
	 *            to transform an object into a list of Strings. XML
	 *            Configuration must be a bean-writer or bean-row XML type, See
	 *            bean-row.xsd file. Must be not <code>null</code>.
	 * @param writer
	 *            Allows to write a list of Strings on a dedicated output. Must
	 *            be not <code>null</code>.
	 * @throws LoadException
	 *             if an error occurs during loading field formatters from file
	 */
	public BeanWriter(File configFile, IWriter writer) throws LoadException {
		this(configFile, writer, false);

	}

	/**
	 * 
	 * @param configFile
	 *            the file which describe how to load the field formatters used
	 *            to transform an object into a list of Strings. XML
	 *            Configuration must be a bean-writer or bean-row XML type, See
	 *            bean-row.xsd file. Must be not <code>null</code>.
	 * @param writer
	 *            Allows to write a list of Strings on a dedicated output. Must
	 *            be not <code>null</code>.
	 * @param skipEmptyRow
	 *            if true avoids writing blank lines.
	 * @throws LoadException
	 *             if an error occurs during loading field formatters from file.
	 */
	public BeanWriter(File configFile, IWriter writer, boolean skipEmptyRow) throws LoadException {
		super();
		if (configFile == null) {
			throw new IllegalArgumentException("BeanWriter Constructor configFile argument must be not null");
		}
		if (writer == null) {
			throw new IllegalArgumentException("BeanWriter Constructor writer argument must be not null");
		}
		if (skipEmptyRow) {
			this.baseWriter = new IgnoreNullWriter(writer);
		} else {
			this.baseWriter = writer;
		}
		this.rowProcessor = new RowProcessor<E>(new CellProcessorLoader<E>().load(configFile));
		if (this.baseWriter.isWithHeaders()) {
			this.baseWriter.addRow(this.rowProcessor.getHeaders());
		}

	}

	/**
	 * Construct a BeanWriter with a rowProcessor and a writer.
	 * 
	 * @param rowProcessor
	 *            the rowProcessor. If the rowProcessor products empty rows,
	 *            blank lines are copied into the file.To avoid this case, use
	 *            IgnoreNullWriter. Must be not <code>null</code>.
	 * @param writer
	 *            the writer.
	 */
	public BeanWriter(IRowProcessor<E> rowProcessor, IWriter writer) {
		super();
		if (rowProcessor == null) {
			throw new IllegalArgumentException("BeanWriter Constructor rowProcessor argument must be not null");
		}
		if (writer == null) {
			throw new IllegalArgumentException("BeanWriter Constructor writer argument must be not null");
		}
		this.baseWriter = writer;
		this.rowProcessor = rowProcessor;
		if (this.baseWriter.isWithHeaders()) {
			this.baseWriter.addRow(this.rowProcessor.getHeaders());
		}
	}

	/**
	 * {@inheritDoc}
	 */
	public void close() {
		try {
			baseWriter.close();
		} catch (IOException e) {
			LOGGER.info("Error on closing IWriter", e);
		}
	}

	/**
	 * Writes a bean on a dedicated output.
	 * <p>
	 * Throws PropertyException if error occurs during property extraction and
	 * formatting.
	 * </p>
	 * 
	 * <p>
	 * Throws WriterException if I/O error occurs during writing.
	 * </p>
	 * 
	 * @param element
	 *            the bean to write. Must be not <code>null</code>.
	 */
	public void write(E element) throws PropertyException {
		if (element == null) {
			throw new IllegalArgumentException("element argument must be not null");
		}

		Row row = rowProcessor.transform(element);
		baseWriter.addRow(row);

	}

	/**
	 * Allows to write a header line to describe the fields.
	 * 
	 * <p>
	 * if the configuration file from which this beanWriter is defined is the
	 * bean-writer element, try to take the declared name of the column
	 * otherwise use "column_"i where i is the column num.
	 * </p>
	 * <p>
	 * if the configuration file from which this beanWriter is defined is the
	 * bean-row element, try to take the name of the declared column, otherwise
	 * use the name of the corresponding property.
	 * 
	 * <p>
	 * throws WriterException if this method is called after objects had been
	 * wroten or I/O error occurs.
	 * </p>
	 */
	public void writeHeaders() {

		baseWriter.addRow(rowProcessor.getHeaders());
	}

}
