/*
 * Copyright 2013 Olivier Godineau
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package olg.csv.bean.annotations.processor;

import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import olg.csv.base.Cell;
import olg.csv.bean.IPropertyProcessor;
import olg.csv.bean.Util;
import olg.csv.bean.annotations.Embedded;
import olg.csv.bean.impl.AbstractPropertyProcessor;
import olg.csv.bean.impl.BeanProcessor;
import olg.csv.bean.impl.CellProcessor;
import olg.csv.bean.impl.PropertyFormatter;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * This class is responsible for processing the @Embedded annotation.
 * 
 * @author Olivier Godineau
 * 
 * @param <B>
 *            the bean on which research its fields annoted with embedded
 *            annotation
 */
public final class EmbeddedProcessor<B> {
	/**
	 * Class logger.
	 */
	private static final Logger LOGGER = LoggerFactory.getLogger(EmbeddedProcessor.class);

	/**
	 * the class of the bean on which research its fields annoted with column
	 * annotation.
	 */
	private Class<B> clazz;

	/**
	 * Constructor.
	 * 
	 * @param clazz
	 *            the class of the bean
	 */
	public EmbeddedProcessor(Class<B> clazz) {
		this.clazz = clazz;
	}

	/**
	 * Complete a map with the cellprocessors built from the embedded annotation
	 * setted on a field.
	 * 
	 * @param field
	 *            the field
	 * @param map
	 *            the map to complete with the cellProcessors built from the
	 *            embedded Annotation setted on the field. the keys are the cell
	 *            num target of the cellProcessor.
	 * @return true if no error happens during this operation.
	 */
	public boolean getWritingProcessor(Field field, Map<Integer, List<CellProcessor<B>>> map) {
		Embedded embedded = field.getAnnotation(Embedded.class);
		boolean noError = true;
		if (embedded != null) {
			String rang = embedded.value();
			int indexBegin = 0;

			try {
				indexBegin = Cell.fromSheetCellNumber(rang);
			} catch (IllegalArgumentException ex) {
				LOGGER.error("{}@Embedded : {}", field.getName(), ex.getMessage());
				noError = false;
			}

			Class<?> fieldConcreteClass = field.getType();
			if (!java.lang.Object.class.equals(embedded.concreteClass())) {
				if (!Util.asSubClass(field.getType(), embedded.concreteClass())) {
					noError = false;
					LOGGER.error("{}@Embedded : {} concreteClass attribute is not a subclass", field.getName(),
							embedded.concreteClass());
				}
				fieldConcreteClass = embedded.concreteClass();

			}
			noError &= checkConcreteClass(field, fieldConcreteClass);

			Method getter = Util.identifyGetter(field.getDeclaringClass(), field.getName());

			if (getter == null) {
				LOGGER.error("{}@Embedded : {} has no public getter for this field", field.getName(), field
						.getDeclaringClass().getSimpleName());
				noError = false;
			}

			Map<Integer, List<CellProcessor<?>>> cellsMap = new HashMap<Integer, List<CellProcessor<?>>>();
			RowBeanProcessor fieldProcessor = new RowBeanProcessor(fieldConcreteClass);
			noError &= fieldProcessor.getWritingProcessor(cellsMap);

			if (noError) {
				for (Entry<Integer, List<CellProcessor<?>>> entry : cellsMap.entrySet()) {
					Integer num = indexBegin + entry.getKey();

					if (map.get(num) == null) {
						map.put(num, new ArrayList<CellProcessor<B>>());
					}

					for (CellProcessor<?> cellProcessor : entry.getValue()) {
						String name = cellProcessor.getName();
						if (Cell.defaultCellName(String.valueOf(cellProcessor.getRang())).equals(name)) {
							name = Cell.defaultCellName(num.toString());
						}
						CellProcessor<B> delegateProcessor = new CellProcessor<B>(num.toString(), name,
								PropertyFormatter.getDelegatePropertyReader(getter, field.getName(),
										cellProcessor.getPropertyFormatter()));
						map.get(num).add(delegateProcessor);
					}
				}
			}
		}
		return noError;
	}

	/**
	 * Complete a list with the propertyProcessors built from the embedded
	 * annotation setted on a field.
	 * 
	 * @param translate
	 *            index from which begin the column numerotation
	 * @param field
	 *            the field on which search an embedded annoation
	 * @param propertyProcessors
	 *            processors map to complete with the processors match on the
	 *            field
	 * @param map
	 *            map of couple (column num, nb properties built on ) to
	 *            complete with the processors built.
	 * @return true if noError occurs.
	 */
	public boolean getReadingProcessor(int translate, Field field, List<IPropertyProcessor<B>> propertyProcessors,
			Map<Integer, Integer> map) {
		Embedded embedded = field.getAnnotation(Embedded.class);
		boolean noError = true;

		int indexBegin = translate;
		if (embedded != null) {
			String rang = embedded.value();

			try {
				indexBegin += Cell.fromSheetCellNumber(rang);
			} catch (IllegalArgumentException ex) {
				LOGGER.error("{}@Embedded : {}", field.getName(), ex.getMessage());
				noError = false;
			}

			Class<?> fieldConcreteClass = field.getType();
			if (!java.lang.Object.class.equals(embedded.concreteClass())) {
				if (!Util.asSubClass(field.getType(), embedded.concreteClass())) {
					noError = false;
					LOGGER.error("{}@Embedded : {} concreteClass attribute is not a subclass", field.getName(),
							embedded.concreteClass());
				}
				fieldConcreteClass = embedded.concreteClass();

			}
			noError &= checkConcreteClass(field, fieldConcreteClass);

			Method setter = Util.identifySetter(field.getDeclaringClass(), field.getName());

			if (setter == null) {
				LOGGER.error("{}@Embedded : {} has no public setter for this field", field.getName(), field
						.getDeclaringClass().getSimpleName());
				noError = false;
			}

			List<IPropertyProcessor<?>> fieldPropertyProcessors = new ArrayList<IPropertyProcessor<?>>();
			RowBeanProcessor fieldProcessor = new RowBeanProcessor(fieldConcreteClass);

			noError &= fieldProcessor.getReadingProcessor(indexBegin, fieldPropertyProcessors, map);

			if (noError) {

				BeanProcessor fieldProcess = new BeanProcessor(fieldConcreteClass);

				fieldProcess.addAll(fieldPropertyProcessors);
				propertyProcessors.add(AbstractPropertyProcessor.getComplexPropertyTransformer(fieldProcess,
						this.clazz, field.getName()));
			}

		}

		return noError;
	}

	/**
	 * check if a field class is concrete.
	 * 
	 * @param field
	 *            the field
	 * @param fieldConcreteClass
	 *            the class used to set the field
	 * @return true if the class is really concrete.
	 */
	private boolean checkConcreteClass(Field field, Class<?> fieldConcreteClass) {
		if (!Util.isConcrete(fieldConcreteClass)) {
			LOGGER.error("{}@Embedded: {}  field class is not a concrete class", field.getName(),
					fieldConcreteClass.getSimpleName());
			return false;
		}
		return true;
	}
}
