/*
 * Copyright 2012 Olivier Godineau
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package olg.csv.bean.impl;

import java.util.ArrayList;
import java.util.List;

import olg.csv.base.Cell;
import olg.csv.base.Row;
import olg.csv.base.WriterException;
import olg.csv.bean.ICellProcessor;
import olg.csv.bean.IRowProcessor;

/**
 * 
 * IRowProcessor implementation.
 * 
 * @author Olivier Godineau
 * 
 * @param <B>
 *            dedicated bean the processor used.
 * 
 */
public final class RowProcessor<B> implements IRowProcessor<B> {

	/**
	 * num of the next row to write.
	 */
	private int nextRowNum = 1;

	/**
	 * Strategy to show values.
	 */
	private final ValueStrategy valueStrategy = new ValueStrategy();
	/**
	 * Strategy to show headers.
	 */
	private final ValueStrategy headerStrategy = new ValueStrategy() {
		@Override
		public Cell getValue(ICellProcessor<B> fieldFormatter, B element) {

			return fieldFormatter.getHeader();
		}
	};

	/**
	 * A rowProcessor has a list of columnFormatter. Each of them converts a
	 * string into a property of the bean this transformer must return.
	 */
	private final List<CellProcessor<B>> fieldFormatters;

	/**
	 * 
	 * @param fieldFormatters
	 *            the list of columnFormatter. Each of them converts a string
	 *            into a property of the bean this processor must return.
	 */
	public RowProcessor(List<CellProcessor<B>> fieldFormatters) {
		super();

		this.fieldFormatters = fieldFormatters;

	}

	/**
	 * {@inheritDoc}
	 */
	public Row transform(B element) {
		return transform(element, valueStrategy);
	}

	/**
	 * {@inheritDoc}
	 */
	public Row getHeaders() {
		if (nextRowNum != 1) {
			throw new WriterException("It's not possible to add headers after objects had been wroten");
		}

		return transform(null, headerStrategy);

	}

	/**
	 * Returns new Row from an element.
	 * 
	 * @param element
	 *            the element.
	 * @param strategy
	 *            the strategy to set the cell values.
	 * @return a new row.
	 */
	private Row transform(B element, ValueStrategy strategy) {
		List<Cell> cells = new ArrayList<Cell>();
		int size = 0;
		for (CellProcessor<B> fieldFormatter : fieldFormatters) {
			size = Math.max(size, fieldFormatter.getRang() + 1);
			cells.add(strategy.getValue(fieldFormatter, element)); // NOPMD
																	// by
																	// olivier
																	// on
																	// 01/02/12
																	// 00:30
		}
		Row row = new Row(nextRowNum++, cells, size);
		return row;
	}
	/**
	 * Strategy to define a cell value.
	 * 
	 * @author Olivier Godineau
	 * 
	 */
	private class ValueStrategy {

		/**
		 * Product a new cell.
		 * 
		 * @param fieldFormatter
		 *            the field formatter to used to product the cell value.
		 * @param element
		 *            the element
		 * @return a new cell
		 */
		public Cell getValue(ICellProcessor<B> fieldFormatter, B element) {
			return fieldFormatter.transform(element);

		}
	}

}
