/*
 * Copyright 2012 Olivier Godineau
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package olg.csv.bean.loader;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpressionException;

import olg.csv.bean.IPropertyProcessor;
import olg.csv.bean.getter.AbstractGetter;
import olg.csv.bean.impl.AbstractPropertyProcessor;
import olg.csv.bean.impl.BeanProcessor;
import olg.csv.bean.loader.getter.AbstractGetterLoader;
import olg.csv.bean.loader.parser.AbstractParserLoader;
import olg.csv.bean.parser.AbstractParser;

import org.w3c.dom.Attr;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

/**
 * Class dedicated to load {@link BeanProcessor} instances from an XML file. XML
 * Root element must be one of the followings :
 * <ul>
 * <li>bean-reader and conform to BeanReaderType from xsd File</li>
 * <li>row-bean and conform to RowBeanType from xsd file</li>
 * </ul>
 * 
 * 
 * @author Olivier Godineau
 * @param <T>
 *            the entity
 * 
 */
public class BeanProcessorLoader<T> {

	/**
	 * 
	 */
	public BeanProcessorLoader() {
		super();
	}

	/**
	 * Returns the transformer described in the given XML file.
	 * 
	 * @param file
	 *            XML file conform to bean-reader xsd declaration. Must be not
	 *            null.
	 * @return a new transformer
	 * @throws LoadException
	 *             if an error occurs during loading
	 */
	public BeanProcessor<T> load(File file) throws LoadException {
		BeanProcessor<T> retour = null;
		if (file == null) {
			throw new IllegalArgumentException("File argument must not be null");
		}
		Document doc;
		try {
			Util.validate(file);
			doc = Util.openDocument(file);
			Element element = (Element) Util.evaluerDOM(doc, "/bean-reader", XPathConstants.NODE);

			if (element == null) {
				element = (Element) Util.evaluerDOM(doc, "/bean-row", XPathConstants.NODE);
				if (element != null) {
					retour = new RowBeanTransformerLoader<T>().getTransformer(element, null);
				}

			} else {

				retour = new BeanProcessorLoader<T>().getTransformer(element, null);
			}

			if (retour != null) {
				return retour;
			}

			throw new LoadException("Root XML Element must be bean-reader or row-bean to create a Bean reader");

		} catch (ParserConfigurationException e) {
			throw new LoadException(e);
		} catch (SAXException e) {
			throw new LoadException(e);
		} catch (IOException e) {
			throw new LoadException(e);
		} catch (XPathExpressionException e) {
			throw new LoadException(e);
		}

	}

	/**
	 * Return thes concrete class with which the given field must be setted.
	 * 
	 * @param name
	 *            the field name
	 * @param className
	 *            a candidate class, if <code>null</code> identify the concrete
	 *            class from its setter
	 * @param beanClass
	 *            the bean class that has the field
	 * @return the concrete class with which set the field
	 * @throws LoadException
	 *             if the class is not concrete
	 * @param <B>
	 *            the bean type that has the field
	 */
	protected <B> Class<?> identifyFieldClass(String name, String className, Class<B> beanClass) throws LoadException {
		Class<?> classField = null;
		if (className == null) {
			// Pas d'attribut class, on choisit la classe du champ tel
			// que
			// déclaré dans le bean
			try {
				classField = olg.csv.bean.Util.identifySetType(beanClass, name);
			} catch (NoSuchMethodException e) {
				throw new LoadException(e);
			}
		} else { // l'attribut class est défini
			try {
				classField = Class.forName(className);
				// TODO s'assurer que la classe est une classe enfant du type du
				// champ

			} catch (ClassNotFoundException e) {
				throw new LoadException("Error with className [" + className + "] : " + e);
			}

		}

		// la classe doit être une classe concrète
		if (!olg.csv.bean.Util.isConcrete(classField)) {
			throw new LoadException(String.format("%s of property[%s] from bean[%s] cannot be instantiated",
					classField, name, beanClass));
		}
		return classField;

	}

	/**
	 * Returns new instance of {@link BeanProcessor} from XML node.
	 * 
	 * @param element
	 *            XML node conform to beanReaderType declaration or beanRowType
	 *            declaration
	 * @param classBean
	 *            the bean class the transformer has responsability to return.
	 *            if <code>null</code> try to load class from class argument of
	 *            the given XML element
	 * @return the bean processor loaded from the XML node.
	 * @throws LoadException
	 *             if an error occurs during loading
	 */
	@SuppressWarnings("unchecked")
	protected final BeanProcessor<T> getTransformer(Element element, Class<T> classBean) throws LoadException {

		try {
			Class<T> beanClass = classBean;
			if (beanClass == null) {
				beanClass = (Class<T>) Class.forName(element.getAttribute("class"));
			}
			if (!Util.checkBean(beanClass)) {
				throw new LoadException(beanClass + " should have been a bean class");
			}
			BeanProcessor<T> transformer = new BeanProcessor<T>(beanClass);
			NodeList liste = (NodeList) Util.evaluerDOM(element, "property", XPathConstants.NODESET);
			for (int i = 0; i < liste.getLength(); i++) {
				transformer.add(getPropertyParser((Element) liste.item(i), beanClass));
				// TODO check unicité déclaration propriété
			}

			return transformer;
		} catch (ClassNotFoundException e) {
			throw new LoadException(e);
		} catch (XPathExpressionException e) {
			throw new LoadException(e);
		}

	}

	/**
	 * Returns new propertyProcessor which have responsability to set a property
	 * to the given bean.
	 * 
	 * @param node
	 *            XML node conform to MappingType declaration
	 * @param beanClass
	 *            parent bean class of the property this propertyTransformer set
	 * @return the property processor loaded from the XML node.
	 * @throws XPathExpressionException
	 *             If an XPath expression cannot be evaluated
	 * @throws LoadException
	 *             if an error occurs during loading
	 * @param <I>
	 *            parent bean type
	 */
	@SuppressWarnings("unchecked")
	protected <I> IPropertyProcessor<I> getPropertyParser(Element node, Class<I> beanClass)
			throws XPathExpressionException, LoadException {
		IPropertyProcessor<I> retour = null;
		// TODO vérifier la classe de la propriété différencier le cas interface
		// abstrait, array List enum...

		String name = node.getAttribute("name");
		String className = Util.emptyToNull(node.getAttribute("class"));

		@SuppressWarnings("rawtypes")
		Class classField = identifyFieldClass(name, className, beanClass);

		AbstractGetter getter = AbstractGetterLoader.getInstance().getGetter(
				(Element) Util.evaluerDOM(node, "getter", XPathConstants.NODE));

		if (getter == null) {
			// La propriété du bean est elle-même un bean dont l'utilisateur
			// veut appliquer des propriétés
			retour = AbstractPropertyProcessor.getComplexPropertyTransformer(getTransformer(node, classField),
					beanClass, name);
		} else { // S'il y a un getter il peut y avoir un parser
			AbstractParser<?> parser = AbstractParserLoader.getInstance().getParser(classField,
					(Element) Util.evaluerDOM(node, "parser", XPathConstants.NODE));

			retour = AbstractPropertyProcessor.getSimplePropertyTransformer(getter, parser, name, beanClass);
		}
		return retour;

	}

	/**
	 * TransformerLoader adaptation in order to manage with beanRowType
	 * elements.
	 * 
	 * @author Olivier Godineau
	 * 
	 * @param <T>
	 */
	protected static final class RowBeanTransformerLoader<T> extends BeanProcessorLoader<T> {

		/**
		 * To check unicity of declared properties.
		 */
		private final List<String> fields = new ArrayList<String>();
		/**
		 * Returns the property processor corresponding to a property node.
		 * 
		 * @param node
		 *            the XML node conform to BeanRowProperty Type XSD
		 *            declaration (property node).
		 * @param beanClass
		 *            the bean which support this property.
		 * @return the property processor corresponding to the property node
		 * @param <I>
		 *            the bean type
		 * @throws XPathExpressionException
		 *             If an XPath expression cannot be evaluated
		 * @throws LoadException
		 *             if an error occurs during loading
		 * 
		 */

		@SuppressWarnings("unchecked")
		@Override
		protected <I> IPropertyProcessor<I> getPropertyParser(Element node, Class<I> beanClass)
				throws XPathExpressionException, LoadException {

			IPropertyProcessor<I> retour = null;
			String name = node.getAttribute("name");
			String className = Util.emptyToNull(node.getAttribute("class"));
			@SuppressWarnings("rawtypes")
			Class classField = identifyFieldClass(name, className, beanClass);
			Element column = (Element) Util.evaluerDOM(node, "column", XPathConstants.NODE);

			if (column == null) {
				// La propriété du bean est elle-même un bean dont l'utilisateur
				// veut appliquer des propriétés
				retour = AbstractPropertyProcessor.getComplexPropertyTransformer(
						new RowBeanTransformerLoader<T>().getTransformer(node, classField), beanClass, name);

			} else {
				String rang = column.getAttribute("rang");
				String defaultValue = null;
				Attr attr = column.getAttributeNode("default");

				if (attr != null) {
					defaultValue = attr.getTextContent();
				}

				if (fields.contains(name)) {
					throw new LoadException(String.format(
							"A property must be mapped only once : Property[%s] is already mapped", name));
				}

				fields.add(name);
				AbstractGetter getter = AbstractGetter.getDefault(rang, defaultValue);

				// S'il y a un getter il peut y avoir un parser
				@SuppressWarnings("rawtypes")
				AbstractParser parser = AbstractParserLoader.getInstance().getParser(classField,
						(Element) Util.evaluerDOM(node, "parser", XPathConstants.NODE));

				retour = AbstractPropertyProcessor.getSimplePropertyTransformer(getter, parser, name, beanClass);
			}
			return retour;
		}

	}

}
