/*
 * Copyright 2012 Olivier Godineau
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package olg.csv.bean.loader;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpressionException;

import olg.csv.base.UsageInvalideException;
import olg.csv.bean.Util;
import olg.csv.bean.loader.parser.AbstractParserLoader;
import olg.csv.bean.parser.AbstractParser;

import org.w3c.dom.Attr;
import org.w3c.dom.DOMException;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

/**
 * Class specialized in bean loading from an XML element conformed to XML schema
 * CustomType specification.
 * 
 * @author Olivier Godineau
 * 
 */
public final class CustomLoader {
	/**
	 * private constructor to prevent its instantiation.
	 */
	private CustomLoader() {

	}
	/**
	 * Returns an instance of a bean corresponding to XML node parameter.
	 * 
	 * @param node
	 *            CustomType node as described in XML schema.
	 * @return a new instance
	 * @throws XPathExpressionException
	 *             on invalid XPathExpression
	 * @throws LoadException
	 *             if bean class not found or instanciation not possible. In
	 *             case of applying properties, a LoadException is thrown when
	 *             property class is not found or setter is not found
	 * @throws XPathExpressionException
	 *             on invalid XPathExpression
	 */
	public static Object getBean(Element node) throws XPathExpressionException, LoadException {
		Object bean = null;

		if (node != null) {
			String className = node.getAttribute("class");
			try {
				bean = Class.forName(className).newInstance();
			} catch (InstantiationException e) {
				throw new LoadException("Error on instanciation " + className, e);
			} catch (IllegalAccessException e) {
				throw new LoadException("Error on instanciation " + className, e);
			} catch (ClassNotFoundException e) {
				throw new LoadException("Error on instanciation " + className, e);
			}
			NodeList liste = (NodeList) olg.csv.bean.loader.Util.evaluerDOM(node, "property", XPathConstants.NODESET);
			for (int i = 0; i < liste.getLength(); i++) {
				setProperty((Element) liste.item(i), bean);

			}
		}
		return bean;
	}

	/**
	 * Sets a property on a bean.
	 * 
	 * @param node
	 *            PropertyType Node as described in XML Schema that define the
	 *            property to set.
	 * 
	 * @param bean
	 *            bean that we want to set the property.
	 * @param <B>
	 *            the type of the bean.
	 * @throws XPathExpressionException
	 *             on invalid XPathExpression
	 * @throws LoadException
	 *             if property class not found or setter not found
	 */

	private static <B> void setProperty(Element node, B bean) throws XPathExpressionException, LoadException {
		if (node != null) {
			String propertyName = node.getAttribute("name");
			Attr attr = node.getAttributeNode("class");
			Class<?> clazz = null;
			AbstractParser<?> parser = null;
			if (attr == null) {
				try {
					clazz = Util.identifySetType(bean.getClass(), propertyName);
				} catch (NoSuchMethodException e) {
					throw new LoadException("Error on setter identification " + propertyName + " from class "
							+ bean.getClass(), e);

				}
			} else {

				try {
					clazz = Class.forName(attr.getTextContent());
				} catch (DOMException e) {
					throw new LoadException("Error on class identification " + attr.getTextContent(), e);
				} catch (ClassNotFoundException e) {
					throw new LoadException("Error on class identification " + attr.getTextContent(), e);
				}
			}

			String value = (String) olg.csv.bean.loader.Util.evaluerDOM(node, "value", XPathConstants.STRING);

			parser = AbstractParserLoader.getInstance().getParser(clazz,
					(Element) olg.csv.bean.loader.Util.evaluerDOM(node, "parser", XPathConstants.NODE));
			if (parser == null) {
				try {
					parser = AbstractParser.identifyDefaultParser(clazz);
				} catch (UsageInvalideException e) {
					throw new LoadException("Error on parser identification used for class " + clazz, e);
				}
			}

			setProperty(parser, propertyName, bean, value);

		}

	}

	/**
	 * Set a property on a bean.
	 * 
	 * @param parser
	 *            the parser to use to parse the value
	 * @param propertyName
	 *            the name of the property
	 * @param bean
	 *            the bean
	 * @param value
	 *            the value
	 * @param <B>
	 *            the type of the bean
	 * @param <T>
	 *            the type of the property
	 * @throws LoadException
	 *             if no property setter exists or error occurs on setter
	 *             invocation.
	 */
	private static <B, T> void setProperty(AbstractParser<T> parser, String propertyName, B bean, String value)
			throws LoadException {
		Method method = olg.csv.bean.Util.identifySetter(bean.getClass(), propertyName);
		if (method == null) {
			throw new LoadException(String.format(
					"Error on setting value[%s] as %s field from bean %s : no method found to set this property",
					value, propertyName, bean.getClass()));
		}
		Object propertyValue = parser.parse(value);
		try {
			method.invoke(bean, new Object[]{propertyValue});
		} catch (IllegalArgumentException e) {
			throw new LoadException(String.format(
					"Error on setting value[%s] as %s field from bean[%s] with invocated method[%s]", value,
					propertyName, bean.getClass(), method.getName()), e);
		} catch (IllegalAccessException e) {
			throw new LoadException(String.format(
					"Error on setting value[%s] as %s field from bean[%s] with invocated method[%s]", value,
					propertyName, bean.getClass(), method.getName()), e);
		} catch (InvocationTargetException e) {
			throw new LoadException(String.format(
					// NOPMD by olivier on 26/02/12 19:15
					"Error on setting value[%s] as %s field from bean[%s] with invocated method[%s]", value,
					propertyName, bean.getClass(), method.getName()), e.getTargetException());

		}
	}

}
