/*
 * Copyright 2012 Olivier Godineau
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package olg.csv.base.csv;

import java.nio.charset.Charset;

/**
 * CSV Settings class to reading and writing.
 * 
 * 
 * @see CSVWriter
 * @see CSVReader
 */
public class CSVSettings {
	/**
	 * Default delimiter.
	 */
	public static final char DEFAULT_DELIMITER = '\"';
	/**
	 * default separator.
	 */
	public static final char DEFAULT_SEPARATOR = ',';

	/**
	 * Default charset of this Java virtual machine.
	 */
	public static final String DEFAULT_CHARSETNAME = Charset.defaultCharset().name();
	/**
	 * Default line separator :operating system line separator.
	 */
	public static final String DEFAULT_LINESEPARATOR = System.getProperty("line.separator");

	/**
	 * Mac Line serapator.
	 */
	public static final String MAC_LINESEPARATOR = "\r";

	/**
	 * Linux line separator.
	 */
	public static final String LINUX_LINESEPARATOR = "\n";
	/**
	 * Dos line separator.
	 */
	public static final String WINDOWS_LINESEPARATOR = "\r\n";

	/**
	 * Default force delimiter.
	 */
	public static final boolean DEFAULT_FORCEDELIMITER = false;
	/**
	 * Default with headers value.
	 */
	public static final boolean DEFAULT_WITHHEADERS = false;
	/**
	 * Default option on skipping empty cell.
	 */
	public static final boolean DEFAULT_SKIPEMPTYCELL = false;

	/**
	 * Cell delimiter. By default {@link CSVSettings#DEFAULT_DELIMITER}.
	 */
	private char delimiter = DEFAULT_DELIMITER;
	/**
	 * Cell separator. By default {@link CSVSettings#DEFAULT_SEPARATOR}.
	 */
	private char separator = DEFAULT_SEPARATOR;

	/**
	 * Charset. By default {@link CSVSettings#DEFAULT_CHARSETNAME}.
	 */
	private String charsetName = DEFAULT_CHARSETNAME;

	/**
	 * Only for writing. By default {@link CSVSettings#DEFAULT_FORCEDELIMITER}.
	 * if true, all cells should be delimited by delimiter character.
	 * 
	 */
	private boolean forceDelimiter = DEFAULT_FORCEDELIMITER;

	/**
	 * By default {@link CSVSettings#DEFAULT_WITHHEADERS}.
	 */
	private boolean withHeaders = DEFAULT_WITHHEADERS;

	/**
	 * Only for reading. By default {@link CSVSettings#DEFAULT_SKIPEMPTYCELL}.
	 * Reading policy: if true the returned rows will only contain not empty
	 * cells.
	 */
	private boolean skipEmptyCell = DEFAULT_SKIPEMPTYCELL;

	/**
	 * line separator. by default {@link CSVSettings#DEFAULT_LINESEPARATOR}.
	 */
	private String lineSeparator = DEFAULT_LINESEPARATOR;

	/**
	 * 
	 * @return the delimiter
	 */
	public char getDelimiter() {
		return delimiter;
	}
	/**
	 * 
	 * @param delimiter
	 *            the delimiter
	 * @return the current settings
	 */
	public CSVSettings setDelimiter(char delimiter) {
		this.delimiter = delimiter;
		return this;
	}
	/**
	 * 
	 * @return the separator.
	 */
	public char getSeparator() {
		return separator;
	}
	/**
	 * 
	 * @param separator
	 *            the separator
	 * @return the current settings
	 */
	public CSVSettings setSeparator(char separator) {
		this.separator = separator;
		return this;
	}

	/**
	 * 
	 * @return the charset name.
	 */
	public String getCharsetName() {
		return charsetName;
	}
	/**
	 * 
	 * @param charsetName
	 *            the charset name
	 * @return the current settings.
	 */
	public CSVSettings setCharsetName(String charsetName) {
		this.charsetName = charsetName;
		return this;
	}
	/**
	 * Indicate if each CSV field must to be delimited.
	 * 
	 * @return if the use of delimiter is forced(CSV writing).
	 */
	public boolean isForceDelimiter() {
		return forceDelimiter;
	}
	/**
	 * 
	 * @param forceDelimiter
	 *            the forceDelimiter setting.
	 * 
	 * @return the current settings.
	 */
	public CSVSettings setForceDelimiter(boolean forceDelimiter) {
		this.forceDelimiter = forceDelimiter;
		return this;
	}

	/**
	 * Indicates if the CSV file has headers.
	 * 
	 * @return the withHeader setting.
	 */
	public boolean isWithHeaders() {
		return withHeaders;
	}

	/**
	 * 
	 * @param withHeaders
	 *            the withHeader setting.
	 * @return the current settings.
	 */
	public CSVSettings setWithHeaders(boolean withHeaders) {
		this.withHeaders = withHeaders;
		return this;
	}

	/**
	 * 
	 * @return if true the returned rows will only contain not empty cells.
	 */
	public boolean isSkipEmptyCell() {
		return skipEmptyCell;
	}
	/**
	 * Only for reading. if true the returned rows will only contain not empty
	 * cells.
	 * 
	 * @param skipEmptyCell
	 *            the skipEmptyCell setting.
	 * @return the current settings.
	 */
	public CSVSettings setSkipEmptyCell(boolean skipEmptyCell) {
		this.skipEmptyCell = skipEmptyCell;
		return this;
	}

	/**
	 * The line separator setting.
	 * 
	 * @return the line separator setting.
	 */
	public String getLineSeparator() {
		return lineSeparator;
	}
	/**
	 * 
	 * @param lineSeparator
	 *            the line separator setting.
	 * @return the current settings.
	 */
	public CSVSettings setLineSeparator(String lineSeparator) {
		this.lineSeparator = lineSeparator;
		return this;
	}
	/**
	 * CSV settings with default values.
	 */
	public CSVSettings() {
		super();

	}

	/**
	 * Defines settings with specified values. Others fields are defined by
	 * default.
	 * 
	 * @param delimiter
	 *            the delimiter
	 * @param separator
	 *            the separator
	 */
	public CSVSettings(char delimiter, char separator) {
		super();
		this.delimiter = delimiter;
		this.separator = separator;

	}

	/**
	 * Defines settings with specified values. Others fields are defined by
	 * default.
	 * 
	 * @param delimiter
	 *            the delimiter
	 * @param separator
	 *            the separator
	 * @param withHeaders
	 *            if the CSV file has headers.
	 */
	public CSVSettings(char delimiter, char separator, boolean withHeaders) {
		super();
		this.delimiter = delimiter;
		this.separator = separator;
		this.withHeaders = withHeaders;

	}

}
