/*
 * Copyright 2012 Olivier Godineau
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package olg.csv.base.csv;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.nio.charset.Charset;
import java.util.Arrays;

import olg.csv.base.Cell;
import olg.csv.base.IWriter;
import olg.csv.base.Row;
import olg.csv.base.WriterException;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * CSV writer. CSV Implementation of IWriter.
 * 
 * @author Olivier Godineau
 * 
 */
public class CSVWriter implements IWriter {

	/**
	 * The logger.
	 */
	private static final Logger LOGGER = LoggerFactory.getLogger(CSVWriter.class);
	/**
	 * Cell delimiter. Default value is {@link CSVSettings#DEFAULT_DELIMITER}.
	 */
	private char delimiter = CSVSettings.DEFAULT_DELIMITER;

	/**
	 * Double Delimiter. Virtue of CSV rfc, delimiter character must be twice in
	 * a cell to represent it.
	 */
	private String ddelimiter = new StringBuilder(CSVSettings.DEFAULT_DELIMITER).append(CSVSettings.DEFAULT_DELIMITER)
			.toString();
	/**
	 * Cell separator. Default value is {@link CSVSettings#DEFAULT_SEPARATOR}.
	 */
	private char separator = CSVSettings.DEFAULT_SEPARATOR;

	/**
	 * Line break. Default value is {@link CSVSettings#DEFAULT_LINESEPARATOR}.
	 */
	private String lineSeparator = CSVSettings.DEFAULT_LINESEPARATOR;
	/**
	 * Charset. Default value is {@link CSVSettings#DEFAULT_CHARSETNAME}.
	 */
	private String charsetName = CSVSettings.DEFAULT_CHARSETNAME;

	/**
	 * indicate if writing an header line is required? Default value is
	 * {@link CSVSettings#DEFAULT_WITHHEADERS}.
	 */
	private boolean withHeaders = CSVSettings.DEFAULT_WITHHEADERS;

	/**
	 * the Output writer.
	 */
	private final OutputStreamWriter outw;

	/**
	 * the out file.
	 */
	private FileOutputStream outFile = null;
	/**
	 * Default value is {@link CSVSettings#DEFAULT_FORCEDELIMITER}.
	 */
	private boolean forceDelimiter = CSVSettings.DEFAULT_FORCEDELIMITER;

	/**
	 * the size of each CSV row.
	 */
	private int size = 0;

	/**
	 * the index of the current line.
	 */
	private int currentLine = 1;

	/**
	 * Constructor.
	 * 
	 * @param out
	 *            the stream on which to write. Must be not <code>null</code>.
	 * @param settings
	 *            the CSV settings to use to write on the stream. Must be not
	 *            <code>null</code> and must have a not <code>null</code>
	 *            lineSeparator setting.
	 * @see CSVSettings for default values
	 */
	public CSVWriter(OutputStream out, CSVSettings settings) {
		if (out == null) {
			throw new IllegalArgumentException("CSVWriter constructor OutputSteam argument must not be null");
		}

		if (settings == null) {
			throw new IllegalArgumentException("CSVWriter constructor CSVSettings argument must not be null");
		}
		if (settings.getLineSeparator() == null || "".equals(settings.getLineSeparator())) {
			throw new IllegalArgumentException("CSVWriter Constructor breakline argument must not be null or empty");
		}

		this.delimiter = settings.getDelimiter();
		this.ddelimiter = new StringBuilder().append(delimiter).append(delimiter).toString();
		this.separator = settings.getSeparator();
		this.lineSeparator = settings.getLineSeparator();
		this.charsetName = settings.getCharsetName();
		this.forceDelimiter = settings.isForceDelimiter();
		this.outw = new OutputStreamWriter(out, Charset.forName(charsetName));
		this.withHeaders = settings.isWithHeaders();

	}
	/**
	 * Constructor.
	 * 
	 * @param outFile
	 *            the file to write. Must be not <code>null</code>.
	 * @param settings
	 *            the CSV settings to use to write on the file. Must be not
	 *            <code>null</code> and have a not <code>null</code>
	 *            lineSeparator setting.
	 * @throws FileNotFoundException
	 *             if the file exists but is a directory rather than a regular
	 *             file, does not exist but cannot be created, or cannot be
	 *             opened for any other reason
	 * @see CSVSettings for default values
	 */
	public CSVWriter(File outFile, CSVSettings settings) throws FileNotFoundException {
		if (outFile == null) {
			throw new IllegalArgumentException("CSVWriter constructor File argument must not be null");
		}

		if (settings == null) {
			throw new IllegalArgumentException("CSVWriter constructor CSVSettings argument must not be null");
		}
		if (settings.getLineSeparator() == null || "".equals(settings.getLineSeparator())) {
			throw new IllegalArgumentException("CSVWriter Constructor breakline argument must not be null or empty");
		}

		this.delimiter = settings.getDelimiter();
		this.ddelimiter = new StringBuilder().append(delimiter).append(delimiter).toString();
		this.separator = settings.getSeparator();
		this.lineSeparator = settings.getLineSeparator();
		this.charsetName = settings.getCharsetName();
		this.forceDelimiter = settings.isForceDelimiter();
		this.withHeaders = settings.isWithHeaders();
		this.outFile = new FileOutputStream(outFile);
		this.outw = new OutputStreamWriter(this.outFile, Charset.forName(charsetName));

	}
	/**
	 * {@inheritDoc}
	 * */
	public void addLine(String[] values) {
		if (values == null || values.length == 0) {
			throw new IllegalArgumentException("CSVWriter#addLine argument must not be null");
		}
		if (size == 0) {
			size = values.length; // when this is first line
		}
		if (size != values.length) {
			throw new WriterException("Values size[" + values.length + "] differs from the expected size :" + size);
		}

		StringBuffer sBuf = new StringBuffer();
		for (String value : values) {
			sBuf.append(formatValue(value)).append(separator);
		}
		try {
			outw.write((currentLine == 0 ? "" : lineSeparator) + sBuf.substring(0, sBuf.length() - 1));
			outw.flush();
			currentLine++;
		} catch (IOException e) {
			throw new WriterException("Write Error at line :" + Arrays.asList(values), e);
		}

	}

	/**
	 * {@inheritDoc}
	 */
	public void addRow(Row row) {
		if (row == null) {
			throw new IllegalArgumentException("CSVWriter#addRow argument must not be null");
		}
		if (size == 0) {
			size = row.getSize(); // when this is first line
		}
		checkRow(row);

		int currentCellNum = 0;
		StringBuilder sBuf = new StringBuilder();

		writeBlankLine(row.getNum());
		for (Cell cell : row) {
			if (cell.getNum() < currentCellNum) {
				throw new IllegalArgumentException("CSVWriter#addRow cells must be ordered ");
			}
			sBuf.append(padding(currentCellNum, cell.getNum()));
			sBuf.append(formatValue(cell.getValue())).append(separator);
			currentCellNum = cell.getNum() + 1;
		}
		sBuf.append(padding(currentCellNum, size));
		try {
			outw.write((currentLine == 1 ? "" : lineSeparator) + sBuf.substring(0, sBuf.length() - 1));
			outw.flush();
			currentLine++;
		} catch (IOException e) {
			throw new WriterException("Write Error at line :" + row.toString(), e);
		}

	}

	/**
	 * Close this stream and caught and logged IOException. if the parent stream
	 * is provided and passed to CSVWriter constructor by user, user must close
	 * it (close what you open!)
	 */
	public void close() {
		if (outw != null) {
			try {
				outw.close();
			} catch (IOException e) {
				LOGGER.info("Error on closing CSVWriter", e);

			}
		}
		if (outFile != null) {
			try {
				outFile.close();
			} catch (IOException e) {
				LOGGER.info("Error on closing CSVWriter fileOutputStream", e);

			}
		}

	}

	/**
	 * add delimiter to a value if necessary or forced.
	 * 
	 * @param value
	 *            the value
	 * @return the value delimited
	 */
	private String formatValue(String value) {
		String retour = value;
		if (value == null) {
			retour = (forceDelimiter ? ddelimiter : "");
		} else {

			if (forceDelimiter || value.indexOf(delimiter) != -1 || value.indexOf(separator) != -1
					|| value.indexOf(lineSeparator) != -1) {
				retour = delimiter + value.replaceAll(String.valueOf(delimiter), ddelimiter) + delimiter;
			}
		}

		return retour;

	}

	/**
	 * writes blank rows until reach a specific row (exclude).
	 * 
	 * @param num
	 *            the row num to reach
	 */
	private void writeBlankLine(int num) {
		StringBuffer sBuf = new StringBuffer();

		for (int i = currentLine; i < num; i++) {
			sBuf.append(lineSeparator);
			sBuf.append(padding(0, size - 1));
			sBuf.append(formatValue(null));
		}
		if (sBuf.length() != 0) {
			try {
				outw.write((currentLine == 1 ? sBuf.substring(lineSeparator.length()) : sBuf.toString()));
				outw.flush();
				currentLine = num;
			} catch (IOException e) {
				throw new WriterException("Blank line Write Error :", e);
			}
		}

	}

	/**
	 * create a stringBuider with end-begin empty csv fields.
	 * 
	 * @param begin
	 *            field index from which beginning the padding.
	 * @param end
	 *            field index
	 * @return the stringBuilder
	 */
	private StringBuilder padding(int begin, int end) {
		StringBuilder sBuf = new StringBuilder();

		for (int num = begin; num < end; num++) {
			sBuf.append(formatValue(null)).append(separator);
		}
		return sBuf;
	}

	/**
	 * check if a row has the correct size and its num is >= the current line.
	 * 
	 * @param row
	 *            the row.
	 */
	private void checkRow(Row row) {

		if (size != row.getSize()) {
			throw new WriterException("Row size[" + row.getSize() + "] differs from the expected size :" + size);
		}
		if (row.getNum() < currentLine) {
			throw new WriterException("Row num[" + row.getNum() + "] but " + currentLine + " was expected or superior");
		}
	}

	/**
	 * {@inheritDoc}
	 */
	public boolean isWithHeaders() {

		return withHeaders;
	}

}
