/*
 * Copyright 2012 Olivier Godineau
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package olg.csv.base.ods;

import java.util.List;

import olg.csv.base.AbstractSheetSettings;
import olg.csv.base.ods.ODSReader.ODSCellFormatter;
import olg.csv.base.ods.ODSWriter.ODSParser;
import olg.csv.bean.formatter.Formatter;
import olg.csv.bean.parser.AbstractParser;

import org.odftoolkit.odfdom.doc.table.OdfTableCell;

/**
 * Settings Class for ODS document reading and writing.
 * 
 * @author Olivier Godineau
 * 
 * @see ODSWriter
 * @see ODSReader
 */
public class ODSSettings extends AbstractSheetSettings<ODSSettings> {
	/**
	 * Default Emptying row value.
	 * <p>
	 * There's two ways to rewrite an ODS file when rows which are passed are
	 * non-consecutive :
	 * <ul>
	 * <li>Intermediate lines are left as</li>
	 * <li>Intermediate lines are emptied (default behavior)</li>
	 * </ul>
	 * </p>
	 * 
	 * @see #setEmptyingRow(boolean)
	 */
	public static final boolean DEFAULT_EMPTYINGROW = true;

	/**
	 * Default Emptying cell value.
	 * <p>
	 * There's two ways to rewrite an ODS file when cells which are passed on a
	 * row are non-consecutive :
	 * <ul>
	 * <li>Intermediate cells are left as</li>
	 * <li>Intermediate cells are emptied</li>
	 * </ul>
	 * </p>
	 * 
	 * @see #setEmptyingCell(boolean)
	 */
	public static final boolean DEFAULT_EMPTYINGCELL = true;

	/**
	 * 
	 */
	private Formatter<OdfTableCell> cellFormatter = null;
	/**
	 * 
	 */
	private AbstractParser<List<String>> stringParser = null;
	/**
	 * Emptying row value. If true intermediate rows will be emptied.
	 */
	private boolean emptyingRow = DEFAULT_EMPTYINGROW;
	/**
	 * Emptying cell value. If true intermediate cells will be emptied.
	 */
	private boolean emptyingCell = DEFAULT_EMPTYINGCELL;

	/**
	 * Purpose default settings.
	 * <ul>
	 * <li>sheetNum = 0</li>
	 * <li>withHeaders = true</li>
	 * <li>beginAtRow = 1</li>
	 * <li>endAtRow = null</li>
	 * <li>beginAtColumn=null</li>
	 * <li><endAtColumn = null</li>
	 * <li>cellFormatter = ODSCellFormatter(\n)</li>
	 * </ul>
	 * 
	 * @see ODSReader.ODSCellFormatter
	 */
	public ODSSettings() {
		super();
		this.cellFormatter = new ODSCellFormatter("\n");
		this.stringParser = new ODSParser();

	}

	/**
	 * Defines settings for writing process.
	 * 
	 * @param sheetName
	 *            The name of the Spreadsheet to write on. If the file on which
	 *            to write exists and contains a sheet with the specified name,
	 *            the writer will write on it otherwise the writer will write on
	 *            a new sheet. If the file doesn't exist, the writer create it
	 *            and its specified sheet.
	 * @param beginAtRow
	 *            specify the first line to begin writing. May be
	 *            <code>null</code>
	 * @param beginAtColumn
	 *            specify the first column where to begin writing. May be
	 *            <code>null</code>
	 * @see ODSReader.ODSCellFormatter ODS Cell Formatter for reading process
	 *      which use'\n' character as line separator
	 * @see ODSParser String Parser used for writing process. Define how to
	 *      interpret a value as a ODS Cell content. For each line returned by
	 *      this parser a paragraph will be created into the cell.
	 */
	public ODSSettings(String sheetName, Integer beginAtRow, String beginAtColumn) {
		super(0, ODSSettings.DEFAULT_WITHHEADERS, beginAtRow, null, beginAtColumn, null, sheetName);
		this.cellFormatter = new ODSCellFormatter("\n");
		this.stringParser = new ODSParser();
	}

	/**
	 * Defines settings for writing process.
	 * 
	 * @param sheetName
	 *            The name of the Spreadsheet to write on. If the file on which
	 *            to write exists and contains a sheet with the specified name,
	 *            the writer will write on it otherwise the writer will write on
	 *            a new sheet. If the file doesn't exist, the writer create it
	 *            and its specified sheet.
	 * @see ODSReader.ODSCellFormatter ODS Cell Formatter for reading process
	 *      which use'\n' character as line separator
	 * @see ODSParser String Parser used for writing process. Define how to
	 *      interpret a value as a ODS Cell content. For each line returned by
	 *      this parser a paragraph will be created into the cell.
	 */
	public ODSSettings(String sheetName) {
		super(0, ODSSettings.DEFAULT_WITHHEADERS, null, null, null, null, sheetName);
		this.cellFormatter = new ODSCellFormatter("\n");
		this.stringParser = new ODSParser();
	}

	/**
	 * Defines settings for Reading process.
	 * 
	 * @param sheetNum
	 *            the table name in the document.First is 0.
	 * @param beginAtRow
	 *            specify the first line to begin reading. May be
	 *            <code>null</code>
	 * @param endAtRow
	 *            specify the last line to read. May be <code>null</code>
	 * @param beginAtColumn
	 *            specify the first column where to begin reading. May be
	 *            <code>null</code>
	 * @param endAtColumn
	 *            specify the last column where to end traitment. May be
	 *            <code>null</code>
	 * @param withHeaders
	 *            indicate if the table has headers.
	 * @see ODSCellFormatter
	 * @see ODSParser
	 */
	public ODSSettings(int sheetNum, Integer beginAtRow, Integer endAtRow, String beginAtColumn, String endAtColumn,
			boolean withHeaders) {
		super(sheetNum, withHeaders, beginAtRow, endAtRow, beginAtColumn, endAtColumn, null);
		this.cellFormatter = new ODSCellFormatter("\n");
		this.stringParser = new ODSParser();
	}

	/**
	 * Defines settings for Reading process.
	 * 
	 * @param sheetNum
	 *            the table name in the document.First is 0.
	 * @param withHeaders
	 *            indicate if the table has headers.
	 * @see ODSCellFormatter
	 * @see ODSParser
	 */
	public ODSSettings(int sheetNum, boolean withHeaders) {
		super(sheetNum, withHeaders, null, null, null, null, null);
		this.cellFormatter = new ODSCellFormatter("\n");
		this.stringParser = new ODSParser();
	}

	/**
	 * Returns a cell formatter. This formatter defines how extract cell content
	 * as a string. Especially when cell is composed with paragraphs or line
	 * breaks; Only for reading.
	 * 
	 * @see ODSReader.ODSCellFormatter
	 * @return the cell formatter.
	 */
	public Formatter<OdfTableCell> getCellFormatter() {
		return cellFormatter;
	}

	/**
	 * Sets the cell formatter to use. This formatter define how extract cell
	 * content as a string. Especially when cell is composed with paragraphs or
	 * line breaks; Only for reading.
	 * 
	 * @param cellFormatter
	 *            the formatter
	 * @return this instance
	 */
	public ODSSettings setCellFormatter(Formatter<OdfTableCell> cellFormatter) {
		this.cellFormatter = cellFormatter;
		return this;
	}

	/**
	 * Returns the String parser. This parser defines how to interpret breakline
	 * characters found into a string and returns lines extracted from this
	 * string. {@link ODSWriter} uses this parser to create for each returned
	 * line a paragraph in the ODS Cell(where line breaks are not interpreted),
	 * target of the original string. Only for writing
	 * 
	 * @see ODSWriter.ODSParser
	 * @return the string parser
	 */
	public AbstractParser<List<String>> getStringParser() {
		return stringParser;
	}

	/**
	 * Set the String parser. This parser defines how to interpret breakline
	 * characters found into a string and returns lines extracted from this
	 * string. {@link ODSWriter} uses this parser to create for each returned
	 * line a paragraph in the ODS Cell target of the original string. Only for
	 * writing
	 * 
	 * @param stringParser
	 *            the parser.
	 * @return this instance
	 * @see ODSWriter.ODSParser
	 */
	public ODSSettings setStringParser(AbstractParser<List<String>> stringParser) {
		this.stringParser = stringParser;
		return this;
	}

	/**
	 * Returns the policy on rows rewriting.
	 * 
	 * @return true if intermediate lines are emptied
	 * @see #setEmptyingRow(boolean)
	 */
	public boolean isEmptyingRow() {
		return emptyingRow;
	}

	/**
	 * Sets the policy on rows rewriting. There's two ways to rewrite an ODS
	 * file when rows which are passed are non-consecutive :
	 * <ul>
	 * <li>Intermediate lines are left as</li>
	 * <li>Intermediate lines are emptied (default behavior)</li>
	 * </ul>
	 * Applied only when an existing file is overridden.
	 * 
	 * @param emptyingRow
	 *            the value to set.
	 * @return the ODSSettings instance.
	 * 
	 */
	public ODSSettings setEmptyingRow(boolean emptyingRow) {
		this.emptyingRow = emptyingRow;
		return this;
	}

	/**
	 * Returns the policy on cells rewriting.
	 * 
	 * @return true if intermediate cells are emptied
	 * @see #setEmptyingCell(boolean)
	 */
	public boolean isEmptyingCell() {
		return emptyingCell;
	}

	/**
	 * Sets the policy on cells rewriting. There's two ways to rewrite an ODS
	 * file when cells which are passed on a row are non-consecutive :
	 * <ul>
	 * <li>Intermediate cells are left as</li>
	 * <li>Intermediate cells are emptied (default behavior)</li>
	 * </ul>
	 * Applied only when an existing file is overridden.
	 * 
	 * @param emptyingCell
	 *            the value to set.
	 * @return this instance
	 * 
	 */
	public ODSSettings setEmptyingCell(boolean emptyingCell) {
		this.emptyingCell = emptyingCell;
		return this;
	}

}
