/*
 * Copyright 2012 Olivier Godineau
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package olg.csv.bean;

import java.io.Closeable;
import java.io.File;
import java.io.IOException;
import java.util.Iterator;

import olg.csv.base.IReader;
import olg.csv.base.IgnoreNullReader;
import olg.csv.base.ReaderException;
import olg.csv.base.Row;
import olg.csv.bean.loader.BeanProcessorLoader;
import olg.csv.bean.loader.LoadException;
import olg.csv.bean.parser.ParseException;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * Class specialized in reading objects from a file. Used a {@link IReader} to
 * read file as iterator of list of Strings. Used a BeanProcessor to transform a
 * list of strings returned by the reader into a new object.
 * 
 * 
 * @author Olivier Godineau
 * @see IReader
 * @see olg.csv.bean.impl.BeanProcessor
 * 
 * @param <E>
 *            object this reader return
 */
public class BeanReader<E> implements Iterator<E>, Closeable {

	/**
	 * The class logger.
	 */
	private static final Logger LOGGER = LoggerFactory.getLogger(BeanReader.class);

	/**
	 * The reader.Allows to read file as iterator of list of Strings.
	 */
	private IReader baseReader = null;

	/**
	 * The beanProcessor. Allows to transform list of Strings into ab object
	 */
	private IBeanProcessor<E> beanProcessor = null;

	/**
	 * {@inheritDoc}
	 */
	public void close() {
		try {
			baseReader.close();
		} catch (IOException e) {
			LOGGER.info("Error on closing IReader", e);
		}

	}
	/**
	 * {@inheritDoc}
	 */
	public boolean hasNext() {
		return baseReader.hasNext();
	}

	/**
	 * {@inheritDoc}
	 */
	public E next() throws ParseException {
		E retour = null;

		Row row = baseReader.next();
		retour = beanProcessor.transform(row);

		return retour;
	}

	/**
	 * @throws UnsupportedOperationException
	 */
	public void remove() {
		throw new UnsupportedOperationException();

	}

	/**
	 * 
	 * @param configFile
	 *            the file which describes how to load the transformer used to
	 *            transform list of String into new object. Must be not
	 *            <code>null</code>.
	 * @param reader
	 *            the reader to use. allows to iterate list of Strings from the
	 *            file the reader deals. Must be not <code>null</code>.
	 * @throws LoadException
	 *             if an error occurs during loading transformer from file
	 */
	public BeanReader(File configFile, IReader reader) throws LoadException {
		this(configFile, reader, false);
	}

	/**
	 * 
	 * @param configFile
	 *            the file which describe how to load the transformer used to
	 *            transform list of String into new object. Must be not
	 *            <code>null</code>.
	 * @param reader
	 *            the reader to use. allows to iterate list of Strings from the
	 *            file the reader deals. Must be not <code>null</code>.
	 * @param skipEmptyRow
	 *            avoids reading blank lines and skip creating beans from them
	 * @throws LoadException
	 *             if an error occurs during loading transformer from file
	 */
	public BeanReader(File configFile, IReader reader, boolean skipEmptyRow) throws LoadException {
		super();
		if (configFile == null) {
			throw new IllegalArgumentException("Constructor configFile argument must be not null");
		}
		if (reader == null) {
			throw new IllegalArgumentException("Constructor reader argument must be not null");
		}
		if (skipEmptyRow) {
			this.baseReader = new IgnoreNullReader(reader);
		} else {
			this.baseReader = reader;
		}

		if (baseReader.isWithHeaders()) {
			if (baseReader.hasNext()) {
				baseReader.next();
			} else {
				throw new ReaderException("No headers");
			}
		}
		this.beanProcessor = new BeanProcessorLoader<E>().load(configFile);
	}

	/**
	 * Constructs a BeanReader with a BeanProcessor and a reader.
	 * 
	 * @param beanProcessor
	 *            the BeanProcessor. Must be not <code>null</code>.
	 * @param reader
	 *            the reader. If this reader products empty row, the Beanreader
	 *            could return empty bean(use IgnoreNullReader to avoid this
	 *            case). Must be not <code>null</code>.
	 */
	public BeanReader(IBeanProcessor<E> beanProcessor, IReader reader) {
		super();
		if (beanProcessor == null) {
			throw new IllegalArgumentException("Constructor beanProcessor argument must be not null");
		}
		if (reader == null) {
			throw new IllegalArgumentException("Constructor reader argument must be not null");
		}
		this.baseReader = reader;
		this.beanProcessor = beanProcessor;
		if (baseReader.isWithHeaders()) {
			if (baseReader.hasNext()) {
				baseReader.next();
			} else {
				throw new ReaderException("No headers");
			}
		}
	}

}
