/*
 * Copyright 2012 Olivier Godineau
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package olg.csv.bean.filter;

import java.util.Locale;

import olg.csv.bean.filter.impl.DecorateFilter;
import olg.csv.bean.filter.impl.LowerCaseFilter;
import olg.csv.bean.filter.impl.ReplaceFilter;
import olg.csv.bean.filter.impl.SplitFilter;
import olg.csv.bean.filter.impl.TrimFilter;
import olg.csv.bean.filter.impl.UpperCaseFilter;
import olg.csv.bean.filter.impl.UpperCaseFirstFilter;

/**
 * Abstract class specialized in converting a string into another.
 * 
 * @author Olivier Godineau
 * 
 */
public abstract class AbstractStringFilter {

	/**
	 * The successor in the filter chain.
	 */
	private AbstractStringFilter filter = null;

	/**
	 * Realizes the specified transformation promised by this filtre.
	 * 
	 * @param value
	 *            the value to filter.
	 * @return the value after filtering.
	 */
	protected abstract String doFiltre(String value);

	/**
	 * Returns the transformed string passed through this filter and his
	 * successors.
	 * 
	 * @param value
	 *            the string to filter
	 * @return the value after successing filtering.
	 */
	public final String filtre(String value) {

		String filtered = doFiltre(value);
		if (filter != null) {
			filtered = filter.filtre(filtered);
		}
		return filtered;
	}

	/**
	 * Returns the successor.
	 * 
	 * @return the successor.
	 */
	public AbstractStringFilter getFilter() {
		return filter;
	}

	/**
	 * Sets the successor.
	 * 
	 * @param filter
	 *            the successor in the filters chain.
	 */
	public void setFilter(AbstractStringFilter filter) {
		this.filter = filter;
	}

	/**
	 * Returns a filter specialized in lowercase setting.
	 * 
	 * @param locale
	 *            the locale to use. if <code>null</code> default locale is
	 *            used.
	 * @return a new instance
	 */
	public static final AbstractStringFilter getLowerFilter(Locale locale) {
		return new LowerCaseFilter(locale);
	}

	/**
	 * Returns a filter specialized in uppercase setting.
	 * 
	 * @param locale
	 *            the locale. If <code>null</code> jvm default locale is used.
	 * @return a new instance
	 */
	public static final AbstractStringFilter getUpperFilter(Locale locale) {
		return new UpperCaseFilter(locale);
	}

	/**
	 * Returns a filter specialized in uppercase setting of the first letter of
	 * a string.
	 * 
	 * @param locale
	 *            the locale. If <code>null</code> jvm default locale is used.
	 * @return a new instance
	 */
	public static final AbstractStringFilter getUpperFirstFilter(Locale locale) {
		return new UpperCaseFirstFilter(locale);
	}

	/**
	 * Returns a filter which decorate the given string.
	 * 
	 * @param before
	 *            this string will be added front of the given string. Can be
	 *            <code>null</code>
	 * @param after
	 *            this string will be added behind the given string. Can be
	 *            <code>null</code>
	 * @return a new instance
	 */
	public static final AbstractStringFilter getDecorateFilter(String before, String after) {
		return new DecorateFilter(after, before);
	}

	/**
	 * Returns a Filter which execute String {@link String#trim()} method.
	 * 
	 * @return a new instance
	 */
	public static final AbstractStringFilter getTrimFilter() {
		return new TrimFilter();
	}

	/**
	 * Returns a filter which extracts a substring by splitting the given string
	 * around matches of an regular expression.
	 * <p>
	 * throws an IllegalArgumentException if separator argument is
	 * <code>null</code>
	 * </p>
	 * 
	 * @param index
	 *            the n-th value
	 * @param separator
	 *            a regular expression. Must be not <code>null</code>
	 * @return a new instance
	 */
	public static final AbstractStringFilter getSplitFilter(int index, String separator) {
		if (separator == null) {
			throw new IllegalArgumentException(
					"AbstractStringFilter#getSplitFilter Argument separator must be not null");
		}
		return new SplitFilter(index, separator);
	}

	/**
	 * Returns a filter which replace all the occurrences of an regular
	 * expression in a string by a particular value.
	 * <p>
	 * throws an IllegalArgumentException if regex or replacement argument is
	 * <code>null</code>
	 * </p>
	 * 
	 * @param regex
	 *            the regular expression
	 * @param replacement
	 *            the replacement value
	 * @return a new instance
	 */
	public static final AbstractStringFilter getReplaceFilter(String regex, String replacement) {
		if (regex == null) {
			throw new IllegalArgumentException("AbstractStringFilter#getReplaceFilter Argument regex must be not null");
		}
		if (replacement == null) {
			throw new IllegalArgumentException(
					"AbstractStringFilter#getReplaceFilter Argument replacement must be not null");
		}
		return new ReplaceFilter(regex, replacement);
	}

}
