/*
 * Copyright 2012 Olivier Godineau
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package olg.csv.bean.filter.impl;

import olg.csv.base.UsageInvalideException;
import olg.csv.bean.filter.AbstractStringFilter;

/**
 * Filter which replaces all the occurrences of an regular expression in a
 * string by a particular value.
 * 
 * @author Olivier Godineau
 * 
 */
public final class ReplaceFilter extends AbstractStringFilter {

	/**
	 * the regular expression.
	 */
	private String regex = null;
	/**
	 * The replacement string.
	 */
	private String replacement = null;

	/**
	 * 
	 * @param regex
	 *            the regular expression
	 * @param replacement
	 *            the replacement value
	 */
	public ReplaceFilter(String regex, String replacement) {
		super();
		this.regex = regex;
		this.replacement = replacement;
	}

	/**
	 * Replaces all occurences matches of {@link #getRegex()} by
	 * {@link #getReplacement()}.
	 * 
	 * <p>
	 * throws UsageInvalideException if the regular expression or the
	 * replacement value is <code>null</code>.
	 * </p>
	 * 
	 * @param value
	 *            the value to filter.
	 * @return the filtered value
	 */
	@Override
	protected String doFiltre(String value) {
		if (regex == null || replacement == null) {
			throw new UsageInvalideException("ReplaceFilter.regex[" + regex + "] Replacefilter.replacement["
					+ replacement + "] must be not null");
		}
		String retour = null;
		if (value != null) {
			retour = value.replaceAll(regex, replacement);
		}
		return retour;
	}
	/**
	 * Regular expression which identify occurrences to be replaced.
	 * 
	 * @return the regexp.
	 */
	public String getRegex() {
		return regex;
	}

	/**
	 * 
	 * @param regex
	 *            the regexp.
	 */
	public void setRegex(String regex) {
		this.regex = regex;
	}
	/**
	 * Replacement value.
	 * 
	 * @return the replacement string.
	 */
	public String getReplacement() {
		return replacement;
	}

	/**
	 * 
	 * @param replacement
	 *            the replacement.
	 */
	public void setReplacement(String replacement) {
		this.replacement = replacement;
	}

}