/*
 * Copyright 2012 Olivier Godineau
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package olg.csv.bean.getter;

import java.util.List;

import olg.csv.base.Row;
import olg.csv.bean.filter.AbstractStringFilter;
import olg.csv.bean.getter.impl.ConcateGetter;
import olg.csv.bean.getter.impl.ConstantGetter;
import olg.csv.bean.getter.impl.DefaultGetter;

/**
 * This class extracts a string from a row. In the bean reading process from a
 * Strings List, this class is used to extract a string identified as property
 * of the bean to produce.
 * 
 * @author Olivier Godineau
 * 
 */
public abstract class AbstractGetter {

	/**
	 * This value is apply when this getter cannot extract String from a row.
	 * <code>null</code> as default value.
	 */
	private String defaultValue = null;

	/**
	 * An optional filter.<code>null</code> as default value.
	 */
	private AbstractStringFilter filter = null;

	/**
	 * In case of this getter cannot extract a string from list, it's possible
	 * to apply a default value in replacement of.
	 * 
	 * @return the default value
	 * @see #doGet(Row)
	 */
	public final String getDefaultValue() {
		return defaultValue;
	}

	/**
	 * Sets the given value as default value.
	 * 
	 * @see AbstractGetter#doGet(Row)
	 * 
	 * @param defaultValue
	 *            the default value to use when the filtered "getted" string is
	 *            <code>null</code> or empty.
	 */
	public final void setDefaultValue(String defaultValue) {
		this.defaultValue = defaultValue;
	}

	/**
	 * Returns the filter used to filter the String returned by
	 * {@link #doGet(Row)}.
	 * 
	 * @return the filter.
	 */
	public final AbstractStringFilter getFilter() {
		return filter;
	}

	/**
	 * Sets the given filter.
	 * 
	 * @param filter
	 *            the fitler.
	 */
	public final void setFilter(AbstractStringFilter filter) {
		this.filter = filter;
	}

	/**
	 * Returns a String built from the given row.
	 * 
	 * @param line
	 *            the row from which extract the string.
	 * @return a String built from the given row
	 */
	protected abstract String doGet(Row line);

	/**
	 * Returns a String from a row. if a filter is defined, the string returned
	 * by doGet method is filtered.If this String is empty or null, the default
	 * value is returned.
	 * <p>
	 * throws an IllegalArgumentException When Error occurs if the given line is
	 * <code>null</code>.
	 * </p>
	 * 
	 * @param line
	 *            the row.
	 * @return the string extracted from the row.
	 */
	public final String get(Row line) {
		if (line == null) {
			throw new IllegalArgumentException("AbstractGetter#get Row argument must be not null");
		}

		String got = (filter == null ? doGet(line) : filter.filtre(doGet(line)));
		return (got == null || "".equals(got) ? defaultValue : got);

	}

	/**
	 * Returns a new instance of the default getter. This getter extracts the
	 * string at the given rank in the row.
	 * 
	 * @param rang
	 *            the cell number in the row. Must be greater or equals than 0
	 *            or conformed to the sheet cell number format.
	 * @param defaultValue
	 *            default value to apply if the filtered String at the given
	 *            rank is <code>null</code> or empty.
	 * @return the default getter.
	 */
	public static final AbstractGetter getDefault(String rang, String defaultValue) {

		AbstractGetter getter = new DefaultGetter(rang);
		getter.setDefaultValue(defaultValue);
		return getter;
	}

	/**
	 * Returns a new instance of a concate Getter. This getter products a string
	 * based on concate strings issued of the givent Getters List.
	 * 
	 * @param getters
	 *            List of AbstractGetters which product a String from a Row.
	 *            Must not be <code>null</code> or empty.
	 * @param defaultValue
	 *            default value to apply if the filtered string from concate is
	 *            <code>null</code> or empty.
	 * @return an instance of a concate getter.
	 */
	public static final AbstractGetter getConcate(List<AbstractGetter> getters, String defaultValue) {
		if (getters == null || getters.isEmpty()) {
			throw new IllegalArgumentException("getConcate getters argument must be not null and not empty");
		}

		return new ConcateGetter(getters, defaultValue);
	}

	/**
	 * Returns a new instance of a constant Getter. This getter always return
	 * the given value regardless of the row.
	 * 
	 * @param defaultValue
	 *            the value the getter returns
	 * @return an instance of a constant getter.
	 */
	public static final AbstractGetter getConstant(String defaultValue) {
		return new ConstantGetter(defaultValue);
	}
}
