/*
 * Copyright 2012 Olivier Godineau
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package olg.csv.bean.impl;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import olg.csv.base.Row;
import olg.csv.bean.IBeanProcessor;
import olg.csv.bean.IPropertyProcessor;
import olg.csv.bean.parser.ParseException;

/**
 * In reading process, this class allows to product beans from list of Strings.
 * 
 * 
 * @author Olivier Godineau
 * @param <B>
 *            dedicated bean the transformer products
 * 
 */
public final class BeanProcessor<B> implements IBeanProcessor<B> {

	/**
	 * bean class this transformer products.
	 */

	private final Class<B> beanClass;

	/**
	 * A BeanProcessor has a list of transformers. Each of them converts a
	 * string into a property of the bean this processor must return.
	 */
	private final List<IPropertyProcessor<B>> properties = new ArrayList<IPropertyProcessor<B>>();

	/**
	 * 
	 * @param beanClass
	 *            the class of beans the transformer returns. This class must be
	 *            conformed to bean class ( has a constructor with no arguments
	 *            and all its properties must be accessible with public getter
	 *            and mutable with public setter)
	 */

	public BeanProcessor(Class<B> beanClass) {

		super();
		this.beanClass = beanClass;

	}

	/**
	 * Adds a concret propertyProcessor.
	 * 
	 * @param arg0
	 *            the propertyProcessor to add.
	 * @return true if added.
	 */
	// TODO tester
	public boolean add(IPropertyProcessor<B> arg0) {
		if (arg0 == null) {
			throw new IllegalArgumentException("BeanProcessor#add : Argument must be not null");
		}

		return properties.add(arg0);
	}

	/**
	 * Adds a propertyProcessor collection.
	 * 
	 * @param arg0
	 *            the collection. Must be not <code>null</code>.
	 * @return true if added
	 */
	public boolean addAll(Collection<IPropertyProcessor<B>> arg0) {
		if (arg0 == null) {
			throw new IllegalArgumentException("BeanProcessor#addAll : Argument must be not null");
		}
		return properties.addAll(arg0);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see olg.csv.bean.IBeanProcessor#transform(olg.csv.base.Row)
	 */
	/**
	 * {@inheritDoc}
	 */
	public B transform(Row line) {
		B bean = null;
		if (line != null && !properties.isEmpty()) {

			try {
				bean = beanClass.newInstance();
			} catch (InstantiationException e) {
				throw new ParseException("Error on instantiation " + beanClass, e);
			} catch (IllegalAccessException e) {
				throw new ParseException("Error on instantiation " + beanClass, e);
			}
			for (IPropertyProcessor<B> property : properties) {
				bean = property.transform(line, bean);

			}
		}

		return bean;
	}

}
