/*
 * Copyright 2012 Olivier Godineau
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package olg.csv.bean.loader;

import java.io.File;
import java.io.IOException;
import java.io.StringWriter;
import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.net.URL;
import java.util.Locale;

import javax.xml.XMLConstants;
import javax.xml.namespace.QName;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;
import javax.xml.validation.Schema;
import javax.xml.validation.SchemaFactory;
import javax.xml.validation.Validator;
import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathExpression;
import javax.xml.xpath.XPathExpressionException;
import javax.xml.xpath.XPathFactory;

import org.w3c.dom.Document;
import org.xml.sax.SAXException;

/**
 * Util in loading objects from XML Documents.
 * 
 * @author Olivier Godineau
 * 
 */
public final class Util { // NOPMD by olivier on 28/01/12 14:41
	/**
	 * private constructor to prevent its instantiation.
	 */
	private Util() {

	}
	/**
	 * Extracts Element from XML Element corresponding to the given XPath
	 * expression.
	 * 
	 * @param document
	 *            The context starting
	 * @param expression
	 *            XPath Expression
	 * @param retour
	 *            returned type
	 * @return the Object that is the result of evaluating the expression and
	 *         converting to the retour
	 * @throws XPathExpressionException
	 *             If the expression cannot be evaluated
	 * 
	 */
	public static Object evaluerDOM(Object document, String expression, QName retour) throws XPathExpressionException {

		// création du XPath
		XPathFactory fabrique = XPathFactory.newInstance();
		XPath xpath = fabrique.newXPath();

		// évaluation de l'expression XPath
		XPathExpression exp = xpath.compile(expression);
		Object resultat = exp.evaluate(document, retour);

		return resultat;

	}

	/**
	 * Validate a file as a bean-row XML file.
	 * 
	 * @param file
	 *            the file to validate.
	 * @throws SAXException
	 *             If any parse errors occur.
	 * @throws IOException
	 *             If any IO errors occur.
	 */
	public static void validate(File file) throws SAXException, IOException {

		URL url = olg.csv.bean.Util.class.getResource("bean-row.xsd");
		Schema schema = SchemaFactory.newInstance(XMLConstants.W3C_XML_SCHEMA_NS_URI).newSchema(url);
		Validator validator = schema.newValidator();
		validator.validate(new StreamSource(file));
	}

	/**
	 * Returns the XML Document corresponding to the content of the given file.
	 * 
	 * @param file
	 *            the file
	 * @return the XML document.
	 * @throws ParserConfigurationException
	 *             if a DocumentBuilder cannot be created which satisfies the
	 *             configuration requested.
	 * @throws SAXException
	 *             If any parse errors occur.
	 * @throws IOException
	 *             If any IO errors occur.
	 */
	public static Document openDocument(File file) throws ParserConfigurationException, SAXException, IOException {

		// création d'une fabrique de documents

		DocumentBuilderFactory fabrique = DocumentBuilderFactory.newInstance();
		// fabrique.setNamespaceAware(true);
		// création d'un constructeur de documents
		DocumentBuilder constructeur = fabrique.newDocumentBuilder();

		Document document = constructeur.parse(file);

		return document;

	}

	/**
	 * transforms a XML Document as a String.
	 * 
	 * @param document
	 *            the document.
	 * @return the string.
	 * @throws TransformerException
	 *             Thrown if it is not possible to create a Transformer instance
	 *             or if an error occurs during the course of the String
	 *             transformation
	 */
	public static String toString(Document document) throws TransformerException {

		DOMSource domSource = new DOMSource(document);
		StringWriter writer = new StringWriter();
		StreamResult result = new StreamResult(writer);
		TransformerFactory tFactory = TransformerFactory.newInstance();
		Transformer transformer = tFactory.newTransformer();
		transformer.transform(domSource, result);
		return writer.toString();
	}

	/**
	 * This test checks only if it has a constructor with no argument and is not
	 * an array or an enum or a primitive.
	 * 
	 * 
	 * @param beanClass
	 *            the class
	 * @param <T>
	 *            the type of the bean.
	 * @return <code>true</code> if the given class is identify as a bean.
	 */
	public static <T> boolean checkBean(Class<T> beanClass) {
		boolean retour = !beanClass.isArray() && !beanClass.isEnum() && !beanClass.isPrimitive();

		if (retour) {
			try { // Tentative d'instanciation
				Constructor<T> constructor = beanClass.getConstructor(new Class[]{});
				constructor.newInstance((Object[]) null);
				retour = true;
			} catch (SecurityException e) {
				retour = false;
			} catch (NoSuchMethodException e) {
				retour = false;
			} catch (IllegalArgumentException e) {
				retour = false;
			} catch (InstantiationException e) {
				retour = false;
			} catch (IllegalAccessException e) {
				retour = false;
			} catch (InvocationTargetException e) {
				retour = false;
			}

		}

		return retour;
	}

	/**
	 * Convert an empty String to <code>null</code>.
	 * 
	 * @param str
	 *            String to convert.
	 * @return the string or null if empty.
	 */
	public static String emptyToNull(String str) {
		if (!"".equals(str)) {
			return str;
		}
		return null;

	}

	/**
	 * Returns a locale corresponding to one of the available locales supported
	 * on the JVM and matches the given parameter.
	 * 
	 * @param locale
	 *            a locale String representation like "en" or "fr_CH"
	 * @return <code>null</code> if none available locale matches with the given
	 *         parameter.
	 * @see Locale
	 */
	public static Locale getLocale(String locale) {

		Locale retour = null;
		if (locale != null && locale.matches("[a-z]{2}(?:_[A-Z]{2})?")) {
			for (Locale availableLocale : Locale.getAvailableLocales()) {
				if (availableLocale.toString().equals(locale)) {
					retour = availableLocale;
					break;
				}
			}
		}

		return retour;
	}
}
