/*
 * Copyright 2012 Olivier Godineau
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package olg.csv.base.csv;

import java.io.ByteArrayInputStream;
import java.io.IOException;

import junit.framework.TestCase;

/**
 * CharSearchReader class Test case.
 * 
 * @author Olivier Godineau
 * 
 */
public class CharSearchReaderTestCase extends TestCase {

	/**
	 * find method test.
	 * 
	 * @throws IOException
	 */
	public void testFind() throws IOException {
		String test = "abcdefghijklmnopqrstuvwxyz-abcdefghijklmnopqrstuvwxyz'abcdefghijklmnopqrstuvwxyz,abcdefghijklmnopqrstuvwxyz";
		CharSearchReader reader = new CharSearchReader(new ByteArrayInputStream(test.getBytes("UTF8")), "UTF8", 30);
		try {

			assertEquals("first char found differs from expected", test.indexOf('-'), reader.find('\'', ',', '-'));
			assertEquals("char found differs from expected", test.indexOf('\''), reader.find('\''));
			assertEquals("char found differs from expected", test.indexOf(','), reader.find(','));
			assertEquals("char found differs from the first time ", test.indexOf('-'), reader.find('-'));
			assertEquals("char found differs from the first time", test.indexOf('\''), reader.find('\''));
		} catch (IOException e) {
			fail(e.getMessage());
		} finally {
			reader.close();
		}
	}

	/**
	 * findNextAtIndex method test.
	 * 
	 * @throws IOException
	 */
	public void testFindNextAtIndex() throws IOException {
		String test = "abcdefghijklmnopqrstuvwxyz-abcdefghijklmnopqrstuvwxyz'abcdefghijklmnopqrstuvwxyz,abcdefghijklmnopqrstuvwxyz";
		CharSearchReader reader = new CharSearchReader(new ByteArrayInputStream(test.getBytes("UTF8")), "UTF8", 30);
		try {

			assertEquals("next char found differs from the expected", test.indexOf('-'),
					reader.findNextAtIndex(test.indexOf('-'), '\'', ',', '-'));
			assertEquals("next char found differs from the expected ", test.indexOf('\''),
					reader.findNextAtIndex(test.indexOf('-') + 1, '\'', ',', '-'));
			assertEquals("next char found differs from the expected", test.indexOf(','),
					reader.findNextAtIndex(test.indexOf('\'') + 1, '\'', ',', '-'));
			assertEquals("next char found differs from the first time", test.indexOf('-'),
					reader.findNextAtIndex(test.indexOf('-'), '\'', ',', '-'));

		} catch (IOException e) {
			fail(e.getMessage());
		} finally {
			reader.close();
		}
	}

	/**
	 * substring method test.
	 * 
	 * @throws IOException
	 */
	public void testSubstring() throws IOException {
		String test = "abcdefghijklmnopqrstuvwxyz-abcdefghijklmnopqrstuvwxyz'abcdefghijklmnopqrstuvwxyz,abcdefghijklmnopqrstuvwxyz";
		CharSearchReader reader = new CharSearchReader(new ByteArrayInputStream(test.getBytes("UTF8")), "UTF8", 30);
		try {
			int index = reader.find('\'', ',', '-');

			assertEquals("substring from 0 differs from the expected", test.substring(0, index),
					reader.substring(0, index));
			assertEquals("substring from 10 differs from the expected", test.substring(10, index),
					reader.substring(10, index));
			int index2 = reader.find('\'');
			assertEquals("substring between specifics index differs from the expected", test.substring(index, index2),
					reader.substring(index, index2));
			assertEquals("substring ending at reader size differs from the expected",
					test.substring(index, reader.getSize()), reader.substring(index, reader.getSize()));

			assertEquals("substring from 0 differs from the first time", test.substring(0, index),
					reader.substring(0, index));

		} catch (IOException e) {
			fail(e.getMessage());
		} finally {
			reader.close();
		}
	}

	/**
	 * skip method test.
	 * 
	 * @throws IOException
	 */
	public void testSkip() throws IOException {
		String test = "abcdefghijklmnopqrstuvwxyz-abcdefghijklmnopqrstuvwxyz'abcdefghijklmnopqrstuvwxyz,abcdefghijklmnopqrstuvwxyz";
		CharSearchReader reader = new CharSearchReader(new ByteArrayInputStream(test.getBytes("UTF8")), "UTF8", 30);
		try {
			int index = reader.find('\'', ',', '-');

			reader.skip(index);
			assertEquals("skip an index character must place this character at 0 ", 0, reader.find('-'));
			index = reader.find(',');
			reader.skip(index);
			assertEquals("skip an index character must place this character at 0 ", 0, reader.find(','));
			char charAt0 = reader.charAt(0);
			reader.skip(0);
			assertEquals("skip 0 characters must not change offset stream ", charAt0, reader.charAt(0));

		} catch (IOException e) {
			fail(e.getMessage());
		} finally {
			reader.close();
		}
	}

	/**
	 * charAt method test.
	 */
	public void testCharAt() throws IOException {
		String test = "abcdefghijklmnopqrstuvwxyz-abcdefghijklmnopqrstuvwxyz'abcdefghijklmnopqrstuvwxyz,abcdefghijklmnopqrstuvwxyz";
		CharSearchReader reader = new CharSearchReader(new ByteArrayInputStream(test.getBytes("UTF8")), "UTF8", 30);
		try {
			// test on cbuf
			assertEquals("charAt(26) differs from expected", '-', reader.charAt(26));
			assertEquals("charAt(53) differs from expected", '\'', reader.charAt(53));
			assertEquals("charAt(80) differs from expected", ',', reader.charAt(80));
			assertEquals("last char found differs from expected", 'z', reader.charAt(106));
			// test on reader store
			assertEquals("charAt(26) differs from the first time", '-', reader.charAt(26));
			assertEquals("charAt(53) differs from the first time", '\'', reader.charAt(53));
			assertEquals("charAt(80) differs from the first time", ',', reader.charAt(80));
			assertEquals("last char found differs from the first time", 'z', reader.charAt(106));
			try {
				reader.charAt(107);
				fail("charAt :out of range must thrown exception");
			} catch (Exception e) {
				assertTrue(e instanceof IOException);
			}

		} catch (IOException e) {
			fail(e.getMessage());
		} finally {
			reader.close();
		}
	}

	/**
	 * exists method test.
	 * 
	 * @throws IOException
	 */
	public void testExists() throws IOException {
		String test = "abcdefghijklmnopqrstuvwxyz-abcdefghijklmnopqrstuvwxyz'abcdefghijklmnopqrstuvwxyz,abcdefghijklmnopqrstuvwxyz";
		CharSearchReader reader = new CharSearchReader(new ByteArrayInputStream(test.getBytes("UTF8")), "UTF8", 30);
		try {
			assertTrue("A character exists at this index", reader.exists(26));
			assertTrue("A character exists at this index", reader.exists(53));
			assertFalse("exists must return false on index out of range", reader.exists(107));
			// test on reader store
			assertTrue("differs from the first time", reader.exists(26));
			assertTrue("differs from the fist time", reader.exists(53));
			assertFalse("exists must return false on index out of range", reader.exists(107));

		} finally {
			reader.close();
		}
	}
}
