/*
 * Copyright 2012 Olivier Godineau
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package olg.csv.base.ods;

import java.io.File;
import java.io.IOException;

import olg.csv.base.AbstractReaderTestCase;
import olg.csv.base.Cell;
import olg.csv.base.Row;

public class ODSReaderTestCase extends AbstractReaderTestCase {

	/**
	 * Test reading sheet Name after loading Sheet identified by its index in
	 * sheets list.
	 */
	public void testgetSheetName() {
		File file = new File(this.getClass()
				.getResource(ODSReaderTestCase.class.getSimpleName() + "-testgetSheetName.ods").getFile());
		ODSSettings settings = new ODSSettings(0, null, null, null, null, false);
		ODSReader reader = null;
		try {
			reader = new ODSReader(file, settings);
			assertEquals("sheet name differs from expeced", "ODSREADERTestCase", reader.getSheetName());
		} catch (IOException e) {
			fail(e.toString());

		} finally {
			if (reader != null) {
				reader.close();
			}
		}

	}

	/**
	 * Reading Test of some date cell
	 */
	public void testDate() {
		File file = new File(this.getClass().getResource(ODSReaderTestCase.class.getSimpleName() + "-testDate.ods")
				.getFile());
		try {
			ODSSettings settings = new ODSSettings(0, null, null, null, null, false);
			ODSReader reader = new ODSReader(file, settings);
			try {
				String message = "date format error";
				assertTrue("first Row expected", reader.hasNext());
				Row row = reader.next();
				assertNotNull("not null row expected", row);
				Cell cell = row.getCell("A");
				assertNotNull("first cell from first row not null expected", cell);
				assertEquals("dd/MM/yy format expected", "15/12/11", cell.getValue());
				cell = row.getCell("B");
				assertNotNull("2nd cell from first row not null expected", cell);
				assertEquals(message, "jeudi 15 décembre 2011", cell.getValue());
				cell = row.getCell("C");
				assertNotNull("3rd cell from first row not null expected", cell);
				assertEquals("dd/MM/yyy hh:mm:ss format expected", "15/12/2011 00:00:00", cell.getValue());
				cell = row.getCell("D");
				assertNotNull("4th cell  not null expected", cell);
				assertEquals(message, "le 15 du mois 12 de l'année 2011", cell.getValue());
				cell = row.getCell("E");
				assertNotNull("5th cell not null expected", cell);
				assertEquals(message, "11-12-15", cell.getValue());
				cell = row.getCell("F");
				assertNotNull("6th cell not null expected", cell);
				assertEquals(message, "50", cell.getValue());
				cell = row.getCell("G");
				assertNotNull("7th cell not null expected", cell);
				assertEquals(message, "12/11", cell.getValue());
				cell = row.getCell("H");
				assertNotNull("8th cell not null expected", cell);
				assertEquals("MMMM date format expected", "décembre", cell.getValue());

			} finally {

				reader.close();

			}
		} catch (IOException e) {
			fail(e.toString());
		}

	}

	/**
	 * reading test of a cell whom vthe value contains severals lines.
	 */
	public void testMultiLineCell() {
		File file = new File(this.getClass()
				.getResource(ODSReaderTestCase.class.getSimpleName() + "-testMultiLineCell.ods").getFile());
		ODSSettings settings = new ODSSettings(0, null, null, null, null, false);
		try {

			ODSReader reader = new ODSReader(file, settings);
			try {

				assertTrue("first row expected", reader.hasNext());

				Row expected = getTemplate(1, "first line\n2nd line\n3rd line");

				Row row = reader.next();
				assertEqualsRow(expected, row);

			} finally {

				reader.close();

			}
		} catch (IOException e) {
			fail(e.toString());
		}
		file = new File(this.getClass()
				.getResource(ODSReaderTestCase.class.getSimpleName() + "-testMultiLineCell2.ods").getFile());

		try {
			ODSReader reader = new ODSReader(file, settings);
			try {

				assertTrue("first row expected", reader.hasNext());

				Row expected = getTemplate(1, "first line\n2nd line\n3rd line");

				Row row = reader.next();
				assertEqualsRow(expected, row);

			} finally {

				reader.close();

			}
		} catch (IOException e) {
			fail(e.toString());
		}
	}

	/**
	 * Reading test lines from an ODS file
	 */
	public void testValid() {
		File file = new File(this.getClass().getResource(ODSReaderTestCase.class.getSimpleName() + "-testValid.ods")
				.getFile());
		ODSSettings settings = new ODSSettings(0, null, null, null, null, false);
		try {
			ODSReader reader = new ODSReader(file, settings);
			try {
				// 1st
				// xxx,yyy,zzz
				assertTrue("first row expected", reader.hasNext());
				Row expected = getTemplate(1, "xxx", "yyy", "zzz");
				Row row = reader.next();
				assertEqualsRow(expected, row);
				// 2nd
				// "123","456","789"
				assertTrue("2nd row expected", reader.hasNext());
				expected = getTemplate(2, "123", "456", "789");
				row = reader.next();
				assertEqualsRow(expected, row);
				// 3rd
				// 11/12/2001,15:25,1368(somme a2 b2 c2)
				assertTrue("3rd row expected", reader.hasNext());
				expected = getTemplate(3, "11/12/2001", "15:25", "1368");
				row = reader.next();
				assertEqualsRow(expected, row);
				// 4th
				// ,,
				assertTrue("4th row expected", reader.hasNext());
				row = reader.next();
				assertEmptyRow(row, 4, false);
				// 5th
				// xxx
				// Xxx,YYY,ZZZ
				assertTrue("5th row expected", reader.hasNext());
				row = reader.next();
				expected = getTemplate(5, "xxx\nXxx", "YYY", "ZZZ");
				assertEqualsRow(expected, row);

				// 6th
				// xxx,,
				assertTrue("6th row expected", reader.hasNext());

				row = reader.next();
				assertNotNull("not null row expected", row);
				assertEquals("Row num error", 6, row.getNum());
				assertEquals("Row size error", 3, row.getSize());

				Cell cell = row.getCell(0);
				assertNotNull("Not null cell expected", cell);
				assertEquals("num cell error", 0, cell.getNum());
				assertFalse("not empty cell expected", cell.isEmpty());
				assertEquals("Cell value error", "xxx", cell.getValue());
				cell = row.getCell(1);
				assertNotNull("Not null cell expected", cell);
				assertEquals("num cell error", 1, cell.getNum());
				assertTrue("Empty cell expected", cell.isEmpty());
				assertNull("null Cell value expected", cell.getValue());

				cell = row.getCell(2);
				assertNotNull("Not null cell expected", cell);
				assertEquals("num cell error", 2, cell.getNum());
				assertTrue("Empty cell expected", cell.isEmpty());
				assertNull("null Cell value expected", cell.getValue());

				assertFalse("End of file expected", reader.hasNext());

			} finally {

				reader.close();

			}
		} catch (IOException e) {
			fail(e.toString());
		}

	}

}
