/*
 * Copyright 2012 Olivier Godineau
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package olg.csv.bean.filter;


import olg.csv.bean.filter.AbstractStringFilter;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import junit.framework.TestCase;

/**
 * AbstractStringFilter Test case
 * 
 * @author Olivier Godineau
 * 
 */
public class AbstractStringFilterTestCase extends TestCase {
	private final static Logger LOGGER = LoggerFactory.getLogger(AbstractStringFilterTestCase.class);

	private static class TestFilter extends AbstractStringFilter {
		private final String test;
		@Override
		protected String doFiltre(String value) {
			String retour = null;
			if (value != null){
				retour = test + value;
			}
			return retour;
		}
		TestFilter(String test) {
			super();
			this.test = test;
		}

	}

	public void testUpperFilter() {
		String upperString = "HÉ _A1O#\t.;?,Ç";
		String lowerString = "hé _a1O#\t.;?,ç";
		String test = "test";
		AbstractStringFilter filtre = AbstractStringFilter.getUpperFilter(null);
		assertNull("Filtering null string must return null",filtre.filtre(null));
		assertEquals("UpperFilter must return an empty String when filters an empty String","", filtre.filtre(""));
		assertEquals("Upper filtering failed",upperString, filtre.filtre(lowerString));
		filtre.setFilter(new TestFilter(test));
		assertEquals("forgot Chained filtering", test+upperString, filtre.filtre(lowerString));
	}

	public void testLowerFilter() {
		String upperString = "HÉ _A1O#\t.;?,Ç";
		String lowerString = "hé _a1o#\t.;?,ç";
		String test = "TEST";
		AbstractStringFilter filtre = AbstractStringFilter.getLowerFilter(null);
		assertNull("Filtering null string must return null",filtre.filtre(null));
		assertEquals("lower filtering failed",lowerString, filtre.filtre(upperString));

		filtre.setFilter(new TestFilter(test));
		assertEquals("forgot Chained filtering",test+lowerString, filtre.filtre(upperString));
	}

	public void testReplaceFilter() {
		String test = "TEST";
		String regexp="\\s+";
		AbstractStringFilter filtre = AbstractStringFilter.getReplaceFilter(regexp, "_");
		assertNull("Filtering null string must return null",filtre.filtre(null));
		assertEquals("Replace filtering failed","truc1_truc2_", filtre.filtre("truc1\ttruc2  "));

		filtre = AbstractStringFilter.getReplaceFilter(test, "_");
		filtre.setFilter(new TestFilter(test));
		assertEquals("Chained filtering  omitted",test+"truc1_truc2  ", filtre.filtre("truc1"+test+"truc2  "));
		try {
			filtre = AbstractStringFilter.getReplaceFilter(regexp, null);

			filtre.filtre(null);
			fail();
		} catch (IllegalArgumentException ex) {
			LOGGER.debug("Expected Error on ReplaceFilter instanciation",ex);
		}
		try {
			filtre = AbstractStringFilter.getReplaceFilter(null, "_");

			filtre.filtre(null);
			fail();
		} catch (IllegalArgumentException ex) {
			LOGGER.debug("Expected Error on ReplaceFilter instanciation",ex);
		}
	}

	public void testSplitFilter() {
		String strToFilter = "truc1 truc2";
		String test = "TEST";
		String regexp="\\s+";
		AbstractStringFilter filtre = AbstractStringFilter.getSplitFilter(0,regexp );
		assertNull("Filtering null string must return null",filtre.filtre(null));
		assertEquals("Split filtering failed","truc1", filtre.filtre(strToFilter));
		filtre = AbstractStringFilter.getSplitFilter(1, regexp);
		assertEquals("Split filtering failed","truc2", filtre.filtre(strToFilter));
		filtre.setFilter(new TestFilter(test));
		assertEquals("Chained filtering  omitted",test+"truc2", filtre.filtre(strToFilter));
		filtre = AbstractStringFilter.getSplitFilter(2, regexp);
		assertEquals("Split filtering failed",null, filtre.filtre(strToFilter));

		try {
			filtre = AbstractStringFilter.getSplitFilter(0, null);
			filtre.filtre(null);
			fail();
		} catch (IllegalArgumentException ex) {
			LOGGER.debug("Expected Error on SplitFilter instanciation",ex);
		}

	}

	public void testDecorateFilter() {
		String test = "TEST";
		String truc = "truc";
		String before = "avant";
		String after = "après";
		String test2 =" est avant ";
		AbstractStringFilter filtre = AbstractStringFilter.getDecorateFilter(null, null);
		assertEquals("Decorate filter (instanciate with null string arguments)must unchange  filtered string",truc, filtre.filtre(truc));
		filtre = AbstractStringFilter.getDecorateFilter("", "");
		assertEquals(" decorate filter(instanciate with empty string arguments) must unchange filtered string",truc, filtre.filtre(truc));
		filtre = AbstractStringFilter.getDecorateFilter("avant", "après");
		assertEquals("decorate filtering failed",before+test2+after, filtre.filtre(test2));
		filtre.setFilter(new TestFilter(test));
		assertEquals("decorate filtering with chained filter failed",test+before+test2+after, filtre.filtre(test2));
		assertNull("Filtering null string must return null",filtre.filtre(null));

	}
	public void testUpperFirstFilter() {
		String upperFirstString = "Hé _a1O#\t.;?,ç";
		String lowerString = "hé _a1O#\t.;?,ç";
		String test = "test";
		AbstractStringFilter filtre = AbstractStringFilter.getUpperFirstFilter(null);
		assertNull("Filtering null string must return null",filtre.filtre(null));
		assertEquals("Upper first filtering failed",upperFirstString, filtre.filtre(lowerString));
		filtre.setFilter(new TestFilter(test));
		assertEquals("Upper first filtering with chained filter failed",test+upperFirstString, filtre.filtre(lowerString));
	}

}
