/*
 * Copyright 2012 Olivier Godineau
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package olg.csv.bean.impl;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

import junit.framework.TestCase;
import olg.csv.base.Row;
import olg.csv.bean.parser.ParseException;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class BeanProcessorTestCase extends TestCase {
	private final static Logger LOGGER = LoggerFactory.getLogger(BeanProcessorTestCase.class);

	private class PropertyTransformer<T> extends AbstractPropertyProcessor<T> {
		private String value = null;

		private Method method;

		public PropertyTransformer(Class<T> beanClass, String value, String propertyName) {
			super();
			this.beanClass = beanClass;
			this.value = value;

			this.method = olg.csv.bean.Util.identifySetter(beanClass, propertyName);
			if (method == null) {
				throw new IllegalArgumentException("No such setter for " + beanClass.getName() + "." + propertyName
						+ " field");
			}
		}

		@Override
		public T transform(Row line, T bean) throws ParseException {
			T retour = bean;
			if (retour == null) {// On instancie le bean
				try {
					retour = beanClass.newInstance();
				} catch (InstantiationException e) {
					throw new ParseException("Error on instanication " + beanClass, e);
				} catch (IllegalAccessException e) {
					throw new ParseException("Error on instanication " + beanClass, e);
				}
			}

			try {// On remplit le champ du bean
				method.invoke(retour, new Object[]{value});
			} catch (IllegalArgumentException e) {
				throw new ParseException("Error on method invocation  " + method.getName(), e);
			} catch (IllegalAccessException e) {
				throw new ParseException("Error on method invocation  " + method.getName(), e);
			} catch (InvocationTargetException e) {
				throw new ParseException("Error on method invocation  " + method.getName(), e);
			}
			return retour;
		}

	}

	private static class TestBean {
		private String field1;
		private String field2;
		public String getField1() {
			return field1;
		}

		@SuppressWarnings("unused")
		public void setField1(String field1) {
			this.field1 = field1;
		}
		public String getField2() {
			return field2;
		}

		@SuppressWarnings("unused")
		public void setField2(String field2) {
			this.field2 = field2;
		}

		@SuppressWarnings("unused")
		public TestBean() {
			super();

		}

	}

	public void testTransformer() {
		BeanProcessor<TestBean> transformer = new BeanProcessor<TestBean>(TestBean.class);
		try {
			assertNull("Transform a null object must product a null object", transformer.transform(null));
			assertNull("Transform an empty row must product a null object if transformer properties is an empty list",
					transformer.transform(new Row(1, null, 5)));
		} catch (ParseException e) {
			LOGGER.error("Parse error", e);
			fail(e.toString());
		}
		try {
			transformer.add(null);
			fail("transformer must not accept null PropertyTransformer");
		} catch (IllegalArgumentException ex) { // NOPMD by olivier on 09/01/12
												// 21:55
		}
		transformer.add(new PropertyTransformer<TestBean>(TestBean.class, "valeur1", "field1"));

		BeanProcessor<TestBean> transformer2 = new BeanProcessor<TestBean>(TestBean.class);
		try {
			transformer2.add(new PropertyTransformer(PropertyTransformer.class, "valeur1", "field1"));
			fail();
		} catch (IllegalArgumentException ex) { // NOPMD by olivier on 09/01/12
												// 21:55

		}

		transformer.add(new PropertyTransformer<TestBean>(TestBean.class, "valeur2", "field2"));

		try {
			assertNull("Transform a null object must product a null object", transformer.transform(null));
		} catch (ParseException e) {
			LOGGER.error("Parse error on TestTransformer", e);
			fail();
		}
		TestBean bean;
		try {
			bean = transformer.transform(new Row(1, null, 5));
			assertEquals("field1 must match specified value", "valeur1", bean.getField1());
			assertEquals("field2 must match specified value", "valeur2", bean.getField2());

		} catch (ParseException e) {
			LOGGER.error("Parse error on TestTransformer", e);
			fail();
		}

	}

}
